@@ -3,6 +3,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <math.h>
+#include <fenv.h>
 #include <string.h>
 #include <signal.h>
 #include <sys/types.h>
@@ -21,6 +22,7 @@
 #include "my_cpuid.h"
 #include "bridge.h"
 #include "signals.h"
+#include "x64shaext.h"
 #ifdef DYNAREC
 #include "custommem.h"
 #include "../dynarec/native_lock.h"
@@ -28,7 +30,11 @@
 
 #include "modrm.h"
 
+#ifdef TEST_INTERPRETER
+uintptr_t Test0F(x64test_t *test, rex_t rex, uintptr_t addr, int *step)
+#else
 uintptr_t Run0F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
+#endif
 {
     uint8_t opcode;
     uint8_t nextop;
@@ -42,27 +48,38 @@ uintptr_t Run0F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
     sse_regs_t *opex, *opgx, eax1;
     mmx87_regs_t *opem, *opgm, eam1;
 
+#ifdef TEST_INTERPRETER
+    x64emu_t *emu = test->emu;
+#endif
     opcode = F8;
 
     switch(opcode) {
 
         case 0x05:                      /* SYSCALL */
+            #ifndef TEST_INTERPRETER
             R_RIP = addr;
             x64Syscall(emu);
+            #endif
             break;
         case 0x06:                      /* CLTS */
             // this is a privilege opcode...
+            #ifndef TEST_INTERPRETER
             emit_signal(emu, SIGSEGV, (void*)R_RIP, 0);
+            #endif
             break;
 
         case 0x08:                      /* INVD */
         case 0x09:                      /* WBINVD */
             // this is a privilege opcode...
+            #ifndef TEST_INTERPRETER
             emit_signal(emu, SIGSEGV, (void*)R_RIP, 0);
+            #endif
             break;
 
         case 0x0B:                      /* UD2 */
+            #ifndef TEST_INTERPRETER
             emit_signal(emu, SIGILL, (void*)R_RIP, 0);
+            #endif
             break;
 
         case 0x0D:
@@ -93,7 +110,7 @@ uintptr_t Run0F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
             nextop = F8;
             GETEX(0);
             GETGX;
-            if((nextop&0xC0)==0xC0)    /* MOVHLPS Gx,Ex */
+            if(MODREG)    /* MOVHLPS Gx,Ex */
                 GX->q[0] = EX->q[1];
             else
                 GX->q[0] = EX->q[0];    /* MOVLPS Gx,Ex */
@@ -135,7 +152,7 @@ uintptr_t Run0F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
             break;
         case 0x18:                       /* PREFETCHh Ed */
             nextop = F8;
-            GETED(0);
+            _GETED(0);
             if(MODREG) {
             } else
             switch((nextop>>3)&7) {
@@ -152,14 +169,16 @@ uintptr_t Run0F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
 
         case 0x1F:                      /* NOP (multi-byte) */
             nextop = F8;
-            GETED(0);
+            FAKEED(0);
             break;
         case 0x20:                      /* MOV REG, crX */
         case 0x21:                      /* MOV REG, drX */
         case 0x22:                      /* MOV cxR, REG */
         case 0x23:                      /* MOV drX, REG */
             // this is a privilege opcode...
+            #ifndef TEST_INTERPRETER
             emit_signal(emu, SIGSEGV, (void*)R_RIP, 0);
+            #endif
             break;
 
         case 0x28:                      /* MOVAPS Gx,Ex */
@@ -194,38 +213,49 @@ uintptr_t Run0F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
             nextop = F8;
             GETEX(0);
             GETGM;
-            if(isnanf(EX->f[1]) || isinff(EX->f[1]) || EX->f[1]>0x7fffffff)
-                GM->sd[1] = 0x80000000;
+            tmp64s = EX->f[1];
+            if (tmp64s==(int32_t)tmp64s && !isnanf(EX->f[1]))
+                GM->sd[1] = (int32_t)tmp64s;
             else
-                GM->sd[1] = EX->f[1];
-            if(isnanf(EX->f[0]) || isinff(EX->f[0]) || EX->f[0]>0x7fffffff)
-                GM->sd[0] = 0x80000000;
+                GM->sd[1] = INT32_MIN;
+            tmp64s = EX->f[0];
+            if (tmp64s==(int32_t)tmp64s && !isnanf(EX->f[0]))
+                GM->sd[0] = (int32_t)tmp64s;
             else
-                GM->sd[0] = EX->f[0];
+                GM->sd[0] = INT32_MIN;
             break;
         case 0x2D:                      /* CVTPS2PI Gm, Ex */
             // rounding should be done; and indefinite integer should also be assigned if overflow or NaN/Inf
             nextop = F8;
             GETEX(0);
             GETGM;
-            for(int i=1; i>=0; --i)
-                if(isnanf(EX->f[i]) || isinff(EX->f[i]) || EX->f[i]>0x7fffffff)
-                    GM->sd[i] = 0x80000000;
+            for(int i=1; i>=0; --i) {
+                if(isnanf(EX->f[i]))
+                    tmp64s = INT32_MIN;
                 else
                     switch(emu->mxcsr.f.MXCSR_RC) {
-                        case ROUND_Nearest:
-                            GM->sd[i] = nearbyintf(EX->f[i]);
+                        case ROUND_Nearest: {
+                            int round = fegetround();
+                            fesetround(FE_TONEAREST);
+                            tmp64s = nearbyintf(EX->f[i]);
+                            fesetround(round);
                             break;
+                        }
                         case ROUND_Down:
-                            GM->sd[i] = floorf(EX->f[i]);
+                            tmp64s = floorf(EX->f[i]);
                             break;
                         case ROUND_Up:
-                            GM->sd[i] = ceilf(EX->f[i]);
+                            tmp64s = ceilf(EX->f[i]);
                             break;
                         case ROUND_Chop:
-                            GM->sd[i] = EX->f[i];
+                            tmp64s = EX->f[i];
                             break;
                     }
+                if (tmp64s==(int32_t)tmp64s)
+                    GM->sd[i] = (int32_t)tmp64s;
+                else
+                    GM->sd[i] = INT32_MIN;
+            }
             break;
         case 0x2E:                      /* UCOMISS Gx, Ex */
             // same for now
@@ -311,6 +341,43 @@ uintptr_t Run0F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
                     }
                     break;
 
+                case 0xC8:  /* SHA1NEXTE Gx, Ex */
+                    nextop = F8;
+                    GETGX;
+                    GETEX(0);
+                    sha1nexte(emu, GX, EX);
+                    break;
+                case 0xC9:  /* SHA1MSG1 Gx, Ex */
+                    nextop = F8;
+                    GETGX;
+                    GETEX(0);
+                    sha1msg1(emu, GX, EX);
+                    break;
+                case 0xCA:  /* SHA1MSG2 Gx, Ex */
+                    nextop = F8;
+                    GETGX;
+                    GETEX(0);
+                    sha1msg2(emu, GX, EX);
+                    break;
+                case 0xCB:  /* SHA256RNDS2 Gx, Ex (, XMM0) */
+                    nextop = F8;
+                    GETGX;
+                    GETEX(0);
+                    sha256rnds2(emu, GX, EX);
+                    break;
+                case 0xCC:  /* SHA256MSG1 Gx, Ex */
+                    nextop = F8;
+                    GETGX;
+                    GETEX(0);
+                    sha256msg1(emu, GX, EX);
+                    break;
+                case 0xCD:  /* SHA256MSG2 Gx, Ex */
+                    nextop = F8;
+                    GETGX;
+                    GETEX(0);
+                    sha256msg2(emu, GX, EX);
+                    break;
+
                 case 0xF0: /* MOVBE Gd, Ed*/
                     nextop = F8;
                     GETGD;
@@ -326,8 +393,12 @@ uintptr_t Run0F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
                     GETED(0);
                     if(rex.w)
                         ED->q[0] = __builtin_bswap64(GD->q[0]);
-                    else
-                        ED->q[0] = __builtin_bswap32(GD->dword[0]);
+                    else {
+                        if(MODREG)
+                            ED->q[0] = __builtin_bswap32(GD->dword[0]);
+                        else
+                            ED->dword[0] = __builtin_bswap32(GD->dword[0]);
+                    }
                     break;
 
                 default:
@@ -356,6 +427,14 @@ uintptr_t Run0F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
                     }
                     break;
 
+                case 0xCC:  /* SHA1RNDS4 Gx, Ex, Ib */
+                    nextop = F8;
+                    GETGX;
+                    GETEX(1);
+                    tmp8u = F8;
+                    sha1rnds4(emu, GX, EX, tmp8u);
+                    break;
+
                 default:
                     return 0;
             }
@@ -785,7 +864,19 @@ uintptr_t Run0F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
             , EB->byte[0]=0;
             ,
         )                               /* 0x90 -> 0x9F SETxx Eb */
-
+        case 0xA0:                      /* PUSH FS */
+            if(rex.is32bits)
+                Push32(emu, emu->segs[_FS]);
+            else
+                Push64(emu, emu->segs[_FS]);
+            break;
+        case 0xA1:                      /* POP FS */
+            if(rex.is32bits)
+                emu->segs[_FS] = Pop32(emu);
+            else
+                emu->segs[_FS] = Pop64(emu);
+            emu->segs_serial[_FS] = 0;
+            break;
         case 0xA2:                      /* CPUID */
             tmp32u = R_EAX;
             my_cpuid(emu, tmp32u);
@@ -800,7 +891,15 @@ uintptr_t Run0F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
             tmp32s >>= (rex.w?6:5);
             if(!MODREG)
             {
+                #ifdef TEST_INTERPRETER
+                test->memaddr=((test->memaddr)+(tmp32s<<(rex.w?3:2)));
+                if(rex.w)
+                    *(uint64_t*)test->mem = *(uint64_t*)test->memaddr;
+                else
+                    *(uint32_t*)test->mem = *(uint32_t*)test->memaddr;
+                #else
                 ED=(reg64_t*)(((uintptr_t)(ED))+(tmp32s<<(rex.w?3:2)));
+                #endif
             }
             if(rex.w) {
                 if(ED->q[0] & (1LL<<tmp8u))
@@ -815,14 +914,10 @@ uintptr_t Run0F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
             }
             break;
         case 0xA4:                      /* SHLD Ed,Gd,Ib */
-        case 0xA5:                      /* SHLD Ed,Gd,CL */
             nextop = F8;
-            GETED((nextop==0xA4)?1:0);
+            GETED(1);
             GETGD;
-            if(opcode==0xA4)
-                tmp8u = F8;
-            else
-                tmp8u = R_CL;
+            tmp8u = F8;
             if(rex.w)
                 ED->q[0] = shld64(emu, ED->q[0], GD->q[0], tmp8u);
             else {
@@ -832,6 +927,34 @@ uintptr_t Run0F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
                     ED->dword[0] = shld32(emu, ED->dword[0], GD->dword[0], tmp8u);
             }
             break;
+        case 0xA5:                      /* SHLD Ed,Gd,CL */
+            nextop = F8;
+            GETED(0);
+            GETGD;
+            tmp8u = R_CL;
+            if(rex.w)
+                ED->q[0] = shld64(emu, ED->q[0], GD->q[0], tmp8u);
+            else {
+                if(MODREG)
+                    ED->q[0] = shld32(emu, ED->dword[0], GD->dword[0], tmp8u);
+                else
+                    ED->dword[0] = shld32(emu, ED->dword[0], GD->dword[0], tmp8u);
+            }
+            break;
+
+        case 0xA8:                      /* PUSH GS */
+            if(rex.is32bits)
+                Push32(emu, emu->segs[_GS]);
+            else
+                Push64(emu, emu->segs[_GS]);
+            break;
+        case 0xA9:                      /* POP GS */
+            if(rex.is32bits)
+                emu->segs[_GS] = Pop32(emu);
+            else
+                emu->segs[_GS] = Pop64(emu);
+            emu->segs_serial[_FS] = 0;
+            break;
 
         case 0xAB:                      /* BTS Ed,Gd */
             CHECK_FLAGS(emu);
@@ -843,7 +966,15 @@ uintptr_t Run0F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
             tmp64s >>= (rex.w?6:5);
             if(!MODREG)
             {
+                #ifdef TEST_INTERPRETER
+                test->memaddr=((test->memaddr)+(tmp64s<<(rex.w?3:2)));
+                if(rex.w)
+                    *(uint64_t*)test->mem = *(uint64_t*)test->memaddr;
+                else
+                    *(uint32_t*)test->mem = *(uint32_t*)test->memaddr;
+                #else
                 ED=(reg64_t*)(((uintptr_t)(ED))+(tmp64s<<(rex.w?3:2)));
+                #endif
             }
             if(rex.w) {
                 if(ED->q[0] & (1LL<<tmp8u))
@@ -889,30 +1020,38 @@ uintptr_t Run0F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
             if((nextop&0xF8)==0xF8) {
                 return addr;            /* SFENCE */
             }
-            GETED(0);
             switch((nextop>>3)&7) {
                 case 0:                 /* FXSAVE Ed */
+                    _GETED(0);
+                    #ifndef TEST_INTERPRETER
                     if(rex.w)
                         fpu_fxsave64(emu, ED);
                     else
                         fpu_fxsave32(emu, ED);
+                    #endif
                     break;
                 case 1:                 /* FXRSTOR Ed */
+                    _GETED(0);
                     if(rex.w)
                         fpu_fxrstor64(emu, ED);
                     else
                         fpu_fxrstor32(emu, ED);
                     break;
                 case 2:                 /* LDMXCSR Md */
+                    GETED(0);
                     emu->mxcsr.x32 = ED->dword[0];
+                    #ifndef TEST_INTERPRETER
                     if(box64_sse_flushto0)
                         applyFlushTo0(emu);
+                    #endif
                     break;
                 case 3:                 /* STMXCSR Md */
+                    GETED(0);
                     ED->dword[0] = emu->mxcsr.x32;
                     break;
                 case 7:                 /* CLFLUSH Ed */
-                    #ifdef DYNAREC
+                    _GETED(0);
+                    #if defined(DYNAREC) && !defined(TEST_INTERPRETER)
                     if(box64_dynarec)
                         cleanDBFromAddressRange((uintptr_t)ED, 8, 0);
                     #endif
@@ -975,7 +1114,15 @@ uintptr_t Run0F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
             tmp64s >>= (rex.w?6:5);
             if(!MODREG)
             {
+                #ifdef TEST_INTERPRETER
+                test->memaddr=((test->memaddr)+(tmp64s<<(rex.w?3:2)));
+                if(rex.w)
+                    *(uint64_t*)test->mem = *(uint64_t*)test->memaddr;
+                else
+                    *(uint32_t*)test->mem = *(uint32_t*)test->memaddr;
+                #else
                 ED=(reg64_t*)(((uintptr_t)(ED))+(tmp64s<<(rex.w?3:2)));
+                #endif
             }
             if(rex.w) {
                 if(ED->q[0] & (1LL<<tmp8u)) {
@@ -1048,6 +1195,8 @@ uintptr_t Run0F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
                             ED->dword[0] ^= (1<<tmp8u);
                             CLEAR_FLAG(F_CF);
                         }
+                        if(MODREG)
+                            ED->dword[1] = 0;
                     }
                     break;
                 case 6:             /* BTR Ed, Ib */
@@ -1068,6 +1217,8 @@ uintptr_t Run0F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
                             ED->dword[0] ^= (1<<tmp8u);
                         } else
                             CLEAR_FLAG(F_CF);
+                        if(MODREG)
+                            ED->dword[1] = 0;
                     }
                     break;
                 case 7:             /* BTC Ed, Ib */
@@ -1088,6 +1239,8 @@ uintptr_t Run0F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
                         else
                             CLEAR_FLAG(F_CF);
                         ED->dword[0] ^= (1<<tmp8u);
+                        if(MODREG)
+                            ED->dword[1] = 0;
                     }
                     break;
 
@@ -1105,7 +1258,15 @@ uintptr_t Run0F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
             tmp64s >>= (rex.w?6:5);
             if(!MODREG)
             {
+                #ifdef TEST_INTERPRETER
+                test->memaddr=((test->memaddr)+(tmp64s<<(rex.w?3:2)));
+                if(rex.w)
+                    *(uint64_t*)test->mem = *(uint64_t*)test->memaddr;
+                else
+                    *(uint32_t*)test->mem = *(uint32_t*)test->memaddr;
+                #else
                 ED=(reg64_t*)(((uintptr_t)(ED))+(tmp64s<<(rex.w?3:2)));
+                #endif
             }
             if(rex.w) {
                 if(ED->q[0] & (1LL<<tmp8u))
@@ -1289,6 +1450,10 @@ uintptr_t Run0F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
             CHECK_FLAGS(emu);
             nextop = F8;
             GETED(0);
+            #ifdef TEST_INTERPRETER
+            test->memsize = 16;
+            ((uint64_t*)test->mem)[1] = ((uint64_t*)test->memaddr)[1];
+            #endif
             switch((nextop>>3)&7) {
                 case 1:
                     if(rex.w) {
@@ -1366,7 +1531,12 @@ uintptr_t Run0F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
             GETGM;
             GM->q = (EM->q > 63) ? 0L : (GM->q >> EM->q);
             break;
-
+        case 0xD4:                   /* PADDQ Gm,Em */
+            nextop = F8;
+            GETEM(0);
+            GETGM;
+            GM->sq += EM->sq;
+            break;
         case 0xD5:                   /* PMULLW Gm,Em */
             nextop = F8;
             GETEM(0);
@@ -1506,7 +1676,7 @@ uintptr_t Run0F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
             break;
         case 0xE7:                   /* MOVNTQ Em,Gm */
             nextop = F8;
-            if((nextop&0xC0)==0xC0)
+            if(MODREG)
                 return 0;
             GETEM(0);
             GETGM;
@@ -1530,7 +1700,13 @@ uintptr_t Run0F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
                 GM->sw[i] = (tmp32s>32767)?32767:((tmp32s<-32768)?-32768:tmp32s);
             }
             break;
-
+        case 0xEA:                  /* PMINSW Gm,Em */
+            nextop = F8;
+            GETEM(0);
+            GETGM;
+            for (int i=0; i<4; ++i)
+                GM->sw[i] = (GM->sw[i]<EM->sw[i])?GM->sw[i]:EM->sw[i];
+            break;
         case 0xEB:                   /* POR Gm, Em */
             nextop = F8;
             GETEM(0);
@@ -1555,7 +1731,13 @@ uintptr_t Run0F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
                 GM->sw[i] = (tmp32s>32767)?32767:((tmp32s<-32768)?-32768:tmp32s);
             }
             break;
-
+        case 0xEE:                  /* PMAXSW Gm,Em */
+            nextop = F8;
+            GETEM(0);
+            GETGM;
+            for (int i=0; i<4; ++i)
+                GM->sw[i] = (GM->sw[i]>EM->sw[i])?GM->sw[i]:EM->sw[i];
+            break;
         case 0xEF:                   /* PXOR Gm, Em */
             nextop = F8;
             GETEM(0);
@@ -1593,7 +1775,12 @@ uintptr_t Run0F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
             GETGM;
             GM->q = (EM->q > 63) ? 0L : (GM->q << EM->ub[0]);
             break;
-
+        case 0xF4:                   /* PMULUDQ Gm,Em */
+            nextop = F8;
+            GETEM(0);
+            GETGM;
+            GM->q = (uint64_t)GM->ud[0] * (uint64_t)EM->ud[0];
+            break;
         case 0xF5:                   /* PMADDWD Gm, Em */
             nextop = F8;
             GETEM(0);
@@ -1614,7 +1801,16 @@ uintptr_t Run0F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
                 tmp32u += (GM->ub[i]>EM->ub[i])?(GM->ub[i] - EM->ub[i]):(EM->ub[i] - GM->ub[i]);
             GM->q = tmp32u;
             break;
-
+        case 0xF7:                   /* MASKMOVQ Gm, Em */
+            nextop = F8;
+            GETEM(0);
+            GETGM;
+            for (int i = 0; i < 8; i++) {
+                if (EM->ub[i] & 0x80) {
+                   ((reg64_t*)(emu->regs[_DI].q[0]))->byte[i] = GM->ub[i];
+                }
+            }
+            break;
         case 0xF8:                   /* PSUBB Gm,Em */
             nextop = F8;
             GETEM(0);
@@ -1636,7 +1832,12 @@ uintptr_t Run0F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
             for(int i=0; i<2; ++i)
                 GM->ud[i] -= EM->ud[i];
             break;
-
+        case 0xFB:                   /* PSUBQ Gm, Em */
+            nextop = F8;
+            GETEM(0);
+            GETGM;
+            GM->sq -= EM->sq;
+            break;
         case 0xFC:                   /* PADDB Gm, Em */
             nextop = F8;
             GETEM(0);
@@ -1663,4 +1864,4 @@ uintptr_t Run0F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
             return 0;
     }
     return addr;
-}
\ No newline at end of file
+}
