@@ -3,6 +3,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <math.h>
+#include <fenv.h>
 #include <string.h>
 #include <signal.h>
 #include <sys/types.h>
@@ -21,29 +22,34 @@
 #include "bridge.h"
 
 #include "modrm.h"
+#include "x64compstrings.h"
 
 static uint8_t ff_mult(uint8_t a, uint8_t b)
 {
 	int retval = 0;
 
 	for(int i = 0; i < 8; i++) {
-		if((b & 1) == 1) 
+		if((b & 1) == 1)
 			retval ^= a;
-		
+
 		if((a & 0x80)) {
 			a <<= 1;
 			a  ^= 0x1b;
 		} else {
 			a <<= 1;
 		}
-		
+
 		b >>= 1;
 	}
-	
+
 	return retval;
 }
 
+#ifdef TEST_INTERPRETER
+uintptr_t Test660F(x64test_t *test, rex_t rex, uintptr_t addr)
+#else
 uintptr_t Run660F(x64emu_t *emu, rex_t rex, uintptr_t addr)
+#endif
 {
     uint8_t opcode;
     uint8_t nextop;
@@ -54,8 +60,9 @@ uintptr_t Run660F(x64emu_t *emu, rex_t rex, uintptr_t addr)
     int32_t tmp32s;
     uint32_t tmp32u;
     uint64_t tmp64u;
-    int64_t tmp64s;
+    int64_t tmp64s, i64[4];
     float tmpf;
+    double tmpd;
     #ifndef NOALIGN
     int is_nan;
     #endif
@@ -106,6 +113,9 @@ uintptr_t Run660F(x64emu_t *emu, rex_t rex, uintptr_t addr)
         0x17, 0x2b, 0x04, 0x7e, 0xba, 0x77, 0xd6, 0x26, 0xe1, 0x69, 0x14, 0x63, 0x55, 0x21, 0x0c, 0x7d,
     };
 
+    #ifdef TEST_INTERPRETER
+    x64emu_t* emu = test->emu;
+    #endif
     opcode = F8;
 
     switch(opcode) {
@@ -124,13 +134,13 @@ uintptr_t Run660F(x64emu_t *emu, rex_t rex, uintptr_t addr)
         break;
     case 0x12:                      /* MOVLPD Gx, Eq */
         nextop = F8;
-        GETED(0);
+        GETE8(0);
         GETGX;
         GX->q[0] = ED->q[0];
         break;
     case 0x13:                      /* MOVLPD Eq, Gx */
         nextop = F8;
-        GETED(0);
+        GETE8(0);
         GETGX;
         ED->q[0] = GX->q[0];
         break;
@@ -149,20 +159,20 @@ uintptr_t Run660F(x64emu_t *emu, rex_t rex, uintptr_t addr)
         break;
     case 0x16:                      /* MOVHPD Gx, Ed */
         nextop = F8;
-        GETED(0);
+        GETE8(0);
         GETGX;
         GX->q[1] = ED->q[0];
         break;
     case 0x17:                      /* MOVHPD Ed, Gx */
         nextop = F8;
-        GETED(0);
+        GETE8(0);
         GETGX;
         ED->q[0] = GX->q[1];
         break;
 
     case 0x1F:                      /* NOP (multi-byte) */
         nextop = F8;
-        GETED(0);
+        FAKEED(0);
         break;
 
     case 0x28:                      /* MOVAPD Gx, Ex */
@@ -186,36 +196,60 @@ uintptr_t Run660F(x64emu_t *emu, rex_t rex, uintptr_t addr)
         GX->d[0] = EM->sd[0];
         GX->d[1] = EM->sd[1];
         break;
-
+    case 0x2B:                      /* MOVNTPD Ex, Gx */
+        nextop = F8;
+        GETEX(0);
+        GETGX;
+        EX->q[0] = GX->q[0];
+        EX->q[1] = GX->q[1];
+        break;
     case 0x2C:                      /* CVTTPD2PI Gm, Ex */
         nextop = F8;
         GETEX(0);
         GETGM;
-        GM->sd[0] = EX->d[0];
-        GM->sd[1] = EX->d[1];
+        tmp64s = EX->d[0];
+        if (tmp64s==(int32_t)tmp64s && !isnan(EX->d[0]))
+            GM->sd[0] = (int32_t)tmp64s;
+        else
+            GM->sd[0] = INT32_MIN;
+        tmp64s = EX->d[1];
+        if (tmp64s==(int32_t)tmp64s && !isnan(EX->d[1]))
+            GM->sd[1] = (int32_t)tmp64s;
+        else
+            GM->sd[1] = INT32_MIN;
         break;
     case 0x2D:                      /* CVTPD2PI Gm, Ex */
         nextop = F8;
         GETEX(0);
         GETGM;
         switch(emu->mxcsr.f.MXCSR_RC) {
-            case ROUND_Nearest:
-                GM->sd[0] = nearbyint(EX->d[0]);
-                GM->sd[1] = nearbyint(EX->d[1]);
+            case ROUND_Nearest: {
+                int round = fegetround();
+                fesetround(FE_TONEAREST);
+                i64[0] = nearbyint(EX->d[0]);
+                i64[1] = nearbyint(EX->d[1]);
+                fesetround(round);
                 break;
+            }
             case ROUND_Down:
-                GM->sd[0] = floor(EX->d[0]);
-                GM->sd[1] = floor(EX->d[1]);
+                i64[0] = floor(EX->d[0]);
+                i64[1] = floor(EX->d[1]);
                 break;
             case ROUND_Up:
-                GM->sd[0] = ceil(EX->d[0]);
-                GM->sd[1] = ceil(EX->d[1]);
+                i64[0] = ceil(EX->d[0]);
+                i64[1] = ceil(EX->d[1]);
                 break;
             case ROUND_Chop:
-                GM->sd[0] = EX->d[0];
-                GM->sd[1] = EX->d[1];
+                i64[0] = EX->d[0];
+                i64[1] = EX->d[1];
                 break;
         }
+        for(int i=0; i<2; ++i)
+            if (i64[i]==(int32_t)i64[i] && !isnan(EX->d[i]))
+                GM->sd[i] = (int32_t)i64[i];
+            else
+                GM->sd[i] = INT32_MIN;
+
         break;
     case 0x2E:                      /* UCOMISD Gx, Ex */
         // no special check...
@@ -316,7 +350,36 @@ uintptr_t Run660F(x64emu_t *emu, rex_t rex, uintptr_t addr)
                         GX->sw[4+i] = EX->sw[i*2+0] - EX->sw[i*2+1];
                 }
                 break;
-
+            case 0x06:  /* PHSUBD Gx, Ex */
+                nextop = F8;
+                GETEX(0);
+                GETGX;
+                for (int i=0; i<2; ++i)
+                    GX->sd[i] = GX->sd[i*2+0] - GX->sd[i*2+1];
+                if(GX == EX) {
+                    GX->q[1] = GX->q[0];
+                } else {
+                    for (int i=0; i<2; ++i)
+                        GX->sd[2+i] = EX->sd[i*2+0] - EX->sd[i*2+1];
+                }
+                break;
+            case 0x07:  /* PHSUBSW Gx, Ex */
+                nextop = F8;
+                GETEX(0);
+                GETGX;
+                for (int i=0; i<4; ++i) {
+                    tmp32s = GX->sw[i*2+0] - GX->sw[i*2+1];
+                    GX->sw[i] = (tmp32s<-32768)?-32768:((tmp32s>32767)?32767:tmp32s);
+                }
+                if(GX == EX) {
+                    GX->q[1] = GX->q[0];
+                } else {
+                    for (int i=0; i<4; ++i) {
+                        tmp32s = EX->sw[i*2+0] - EX->sw[i*2+1];
+                        GX->sw[4+i] = (tmp32s<-32768)?-32768:((tmp32s>32767)?32767:tmp32s);
+                    }
+                }
+                break;
             case 0x08:  /* PSIGNB Gx, Ex */
                 nextop = F8;
                 GETEX(0);
@@ -367,14 +430,27 @@ uintptr_t Run660F(x64emu_t *emu, rex_t rex, uintptr_t addr)
                         GX->ud[i] = EX->ud[i];
                 }
                 break;
+            case 0x15:  /* BLENDVPD Gx, Ex */
+                nextop = F8;
+                GETEX(0);
+                GETGX;
+                for (int i=0; i<2; ++i) {
+                    if(emu->xmm[0].q[i]&0x8000000000000000LL)
+                        GX->q[i] = EX->q[i];
+                }
+                break;
 
             case 0x17:      // PTEST GX, EX
                 nextop = F8;
                 GETEX(0);
                 GETGX;
-                CHECK_FLAGS(emu);
+                RESET_FLAGS(emu);
                 CONDITIONAL_SET_FLAG(!(GX->u128&EX->u128), F_ZF);
                 CONDITIONAL_SET_FLAG(!((~GX->u128)&EX->u128), F_CF);
+                CLEAR_FLAG(F_AF);
+                CLEAR_FLAG(F_OF);
+                CLEAR_FLAG(F_SF);
+                CLEAR_FLAG(F_PF);
                 break;
 
             case 0x1C:  /* PABSB Gx, Ex */
@@ -444,7 +520,41 @@ uintptr_t Run660F(x64emu_t *emu, rex_t rex, uintptr_t addr)
                 for(int i=1; i>=0; --i)
                     GX->sq[i] = EX->sd[i];
                 break;
-            
+
+            case 0x28:  /* PMULDQ Gx, Ex */
+                nextop = F8;
+                GETEX(0);
+                GETGX;
+                GX->sq[1] = ((int64_t)GX->sd[2])*(int64_t)EX->sd[2];
+                GX->sq[0] = ((int64_t)GX->sd[0])*(int64_t)EX->sd[0];
+                break;
+            case 0x29:  /* PCMPEQQ Gx, Ex */
+                nextop = F8;
+                GETEX(0);
+                GETGX;
+                for(int i=1; i>=0; --i)
+                    GX->sq[i] = (GX->sq[i]==EX->sq[i])?-1LL:0LL;
+                break;
+            case 0x2A:  /* MOVNTDQA Gx, Ex */
+                nextop = F8;
+                GETEX(0);
+                GETGX;
+                GX->q[0] = EX->q[0];
+                GX->q[1] = EX->q[1];
+                break;
+            case 0x2B:  /* PACKUSDW Gx, Ex */
+                nextop = F8;
+                GETEX(0);
+                GETGX;
+                for(int i=0; i<4; ++i)
+                    GX->uw[i] = (GX->sd[i]<0)?0:((GX->sd[i]>65535)?65535:GX->sd[i]);
+                if(GX==EX)
+                    GX->q[1] = GX->q[0];
+                else
+                    for(int i=0; i<4; ++i)
+                        GX->uw[i+4] = (EX->sd[i]<0)?0:((EX->sd[i]>65535)?65535:EX->sd[i]);
+                break;
+
             case 0x30: /* PMOVZXBW Gx, Ex */
                 nextop = F8;
                 GETEX(0);
@@ -488,6 +598,13 @@ uintptr_t Run660F(x64emu_t *emu, rex_t rex, uintptr_t addr)
                     GX->q[i] = EX->ud[i];
                 break;
 
+            case 0x37: /* PCMPGTQ Gx, Ex */
+                nextop = F8;
+                GETEX(0);
+                GETGX;
+                for(int i=1; i>=0; --i)
+                    GX->sq[i] = (GX->sq[i]>EX->sq[i])?-1LL:0LL;
+                break;
             case 0x38:  /* PMINSB Gx, Ex */
                 nextop = F8;
                 GETEX(0);
@@ -552,6 +669,30 @@ uintptr_t Run660F(x64emu_t *emu, rex_t rex, uintptr_t addr)
                     if(GX->ud[i]<EX->ud[i])
                         GX->ud[i] = EX->ud[i];
                 break;
+            case 0x40:  /* PMULLD Gx, Ex */
+                nextop = F8;
+                GETEX(0);
+                GETGX;
+                for(int i=0; i<4; ++i)
+                    GX->ud[i] *= EX->ud[i];
+                break;
+            case 0x41:  /* PHMINPOSUW Gx, Ex */
+                nextop = F8;
+                GETEX(0);
+                GETGX;
+                tmp16u = EX->uw[0];
+                tmp16s = 0;
+                for(int i=1; i<8; ++i) {
+                    if(EX->uw[i]<tmp16u) {
+                        tmp16u = EX->uw[i];
+                        tmp16s = i;
+                    }
+                }
+                GX->q[1] = 0;
+                GX->uw[0] = tmp16u;
+                GX->uw[1] = tmp16s;
+                GX->ud[1] = 0;
+                break;
 
             case 0xDB:  /* AESIMC Gx, Ex */
                 nextop = F8;
@@ -654,7 +795,18 @@ uintptr_t Run660F(x64emu_t *emu, rex_t rex, uintptr_t addr)
                 GX->q[0] ^= EX->q[0];
                 GX->q[1] ^= EX->q[1];
                 break;
-
+            case 0xF0: /* MOVBE Gw, Ew */
+                nextop = F8;
+                GETED(0);
+                GETGD;
+                GD->word[0] = __builtin_bswap16(ED->word[0]);
+                break;
+            case 0xF1: /* MOVBE Ew, Gw */
+                nextop = F8;
+                GETED(0);
+                GETGD;
+                ED->word[0] = __builtin_bswap16(GD->word[0]);
+                break;
             default:
                 return 0;
         }
@@ -663,6 +815,70 @@ uintptr_t Run660F(x64emu_t *emu, rex_t rex, uintptr_t addr)
     case 0x3A:  // these are some SSE3 & SSE4.x opcodes
         opcode = F8;
         switch(opcode) {
+            case 0x08:          // ROUNDPS Gx, Ex, u8
+                nextop = F8;
+                GETEX(1);
+                GETGX;
+                tmp8u = F8; // ignoring bit 3 interupt thingy
+                if(tmp8u&4)
+                    tmp8u = emu->mxcsr.f.MXCSR_RC;
+                else
+                    tmp8u &= 3;
+                switch(tmp8u) {
+                    case ROUND_Nearest: {
+                        int round = fegetround();
+                        fesetround(FE_TONEAREST);
+                        for(int i=0; i<4; ++i)
+                            GX->f[i] = nearbyintf(EX->f[i]);
+                        fesetround(round);
+                        break;
+                    }
+                    case ROUND_Down:
+                        for(int i=0; i<4; ++i)
+                            GX->f[i] = floorf(EX->f[i]);
+                        break;
+                    case ROUND_Up:
+                        for(int i=0; i<4; ++i)
+                            GX->f[i] = ceilf(EX->f[i]);
+                        break;
+                    case ROUND_Chop:
+                        for(int i=0; i<4; ++i)
+                            GX->f[i] = truncf(EX->f[i]);
+                        break;
+                }
+                break;
+            case 0x09:          // ROUNDPD Gx, Ex, u8
+                nextop = F8;
+                GETEX(1);
+                GETGX;
+                tmp8u = F8; // ignoring bit 3 interupt thingy
+                if(tmp8u&4)
+                    tmp8u = emu->mxcsr.f.MXCSR_RC;
+                else
+                    tmp8u &= 3;
+                switch(tmp8u) {
+                    case ROUND_Nearest: {
+                        int round = fegetround();
+                        fesetround(FE_TONEAREST);
+                        GX->d[0] = nearbyint(EX->d[0]);
+                        GX->d[1] = nearbyint(EX->d[1]);
+                        fesetround(round);
+                        break;
+                    }
+                    case ROUND_Down:
+                        GX->d[0] = floor(EX->d[0]);
+                        GX->d[1] = floor(EX->d[1]);
+                        break;
+                    case ROUND_Up:
+                        GX->d[0] = ceil(EX->d[0]);
+                        GX->d[1] = ceil(EX->d[1]);
+                        break;
+                    case ROUND_Chop:
+                        GX->d[0] = trunc(EX->d[0]);
+                        GX->d[1] = trunc(EX->d[1]);
+                        break;
+                }
+                break;
             case 0x0A:          // ROUNDSS Gx, Ex, u8
                 nextop = F8;
                 GETEX(1);
@@ -673,9 +889,13 @@ uintptr_t Run660F(x64emu_t *emu, rex_t rex, uintptr_t addr)
                 else
                     tmp8u &= 3;
                 switch(tmp8u) {
-                    case ROUND_Nearest:
+                    case ROUND_Nearest: {
+                        int round = fegetround();
+                        fesetround(FE_TONEAREST);
                         GX->f[0] = nearbyintf(EX->f[0]);
+                        fesetround(round);
                         break;
+                    }
                     case ROUND_Down:
                         GX->f[0] = floorf(EX->f[0]);
                         break;
@@ -683,7 +903,7 @@ uintptr_t Run660F(x64emu_t *emu, rex_t rex, uintptr_t addr)
                         GX->f[0] = ceilf(EX->f[0]);
                         break;
                     case ROUND_Chop:
-                        GX->f[0] = EX->f[0];
+                        GX->f[0] = truncf(EX->f[0]);
                         break;
                 }
                 break;
@@ -697,9 +917,13 @@ uintptr_t Run660F(x64emu_t *emu, rex_t rex, uintptr_t addr)
                 else
                     tmp8u &= 3;
                 switch(tmp8u) {
-                    case ROUND_Nearest:
+                    case ROUND_Nearest: {
+                        int round = fegetround();
+                        fesetround(FE_TONEAREST);
                         GX->d[0] = nearbyint(EX->d[0]);
+                        fesetround(round);
                         break;
+                    }
                     case ROUND_Down:
                         GX->d[0] = floor(EX->d[0]);
                         break;
@@ -707,11 +931,31 @@ uintptr_t Run660F(x64emu_t *emu, rex_t rex, uintptr_t addr)
                         GX->d[0] = ceil(EX->d[0]);
                         break;
                     case ROUND_Chop:
-                        GX->d[0] = EX->d[0];
+                        GX->d[0] = trunc(EX->d[0]);
                         break;
                 }
                 break;
 
+            case 0x0C:  /* PBLENDPS Gx, Ex, Ib */
+                nextop = F8;
+                GETEX(1);
+                GETGX;
+                tmp8u = F8;
+                for (int i=0; i<4; ++i) {
+                    if(tmp8u&(1<<i))
+                        GX->ud[i] = EX->ud[i];
+                }
+                break;
+            case 0x0D:  /* PBLENDPD Gx, Ex, Ib */
+                nextop = F8;
+                GETEX(1);
+                GETGX;
+                tmp8u = F8;
+                for (int i=0; i<2; ++i) {
+                    if(tmp8u&(1<<i))
+                        GX->q[i] = EX->q[i];
+                }
+                break;
             case 0x0E:  /* PBLENDW Gx, Ex, Ib */
                 nextop = F8;
                 GETEX(1);
@@ -738,14 +982,26 @@ uintptr_t Run660F(x64emu_t *emu, rex_t rex, uintptr_t addr)
                 }
                 break;
 
-            case 0x14:      // PEXTRB EB, GX, u8
+            case 0x14:      // PEXTRB ED, GX, u8
                 nextop = F8;
-                GETEB(1);
+                GETED(1);
                 GETGX;
                 tmp8u = F8;
-                EB->byte[0] = GX->ub[tmp8u&0x0f];
+                if(MODREG)
+                    ED->q[0] = GX->ub[tmp8u&0x0f];
+                else
+                    ED->byte[0] = GX->ub[tmp8u&0x0f];
+                break;
+            case 0x15:      // PEXTRW Ew,Gx,Ib
+                nextop = F8;
+                GETED(1);
+                GETGX;
+                tmp8u = F8;
+                if(MODREG)
+                    ED->q[0] = GX->uw[tmp8u&7];  // 16bits extract, 0 extended
+                else
+                    ED->word[0] = GX->uw[tmp8u&7];
                 break;
-
             case 0x16:      // PEXTRD/Q ED, GX, u8
                 nextop = F8;
                 GETED(1);
@@ -754,7 +1010,10 @@ uintptr_t Run660F(x64emu_t *emu, rex_t rex, uintptr_t addr)
                 if(rex.w) {
                     ED->q[0] = GX->q[tmp8u&1];
                 } else {
-                    ED->q[0] = GX->ud[tmp8u&3];
+                    if(MODREG)
+                        ED->q[0] = GX->ud[tmp8u&3];
+                    else
+                        ED->dword[0] = GX->ud[tmp8u&3];
                 }
                 break;
             case 0x17:      // EXTRACTPS ED, GX, u8
@@ -807,6 +1066,39 @@ uintptr_t Run660F(x64emu_t *emu, rex_t rex, uintptr_t addr)
                 for(int i=0; i<4; ++i)
                     GX->f[i] = (tmp8u&(1<<i))?tmpf:0.0f;
                 break;
+            case 0x41:  /* DPPD Gx, Ex, Ib */
+                nextop = F8;
+                GETEX(1);
+                GETGX;
+                tmp8u = F8;
+                tmpd = 0.0;
+                if(tmp8u&(1<<(4+0)))
+                    tmpd += GX->d[0]*EX->d[0];
+                if(tmp8u&(1<<(4+1)))
+                    tmpd += GX->d[1]*EX->d[1];
+                GX->d[0] = (tmp8u&(1<<(0)))?tmpd:0.0;
+                GX->d[1] = (tmp8u&(1<<(1)))?tmpd:0.0;
+                break;
+            case 0x42:  /* MPSADBW Gx, Ex, Ib */
+                nextop = F8;
+                GETEX(1);
+                GETGX;
+                tmp8u = F8;
+                {
+                    int src = tmp8u&3;
+                    int dst = (tmp8u>>2)&1;
+                    int b[11];
+                    for (int i=0; i<11; ++i)
+                        b[i] = GX->ub[dst*4+i];
+                    for(int i=0; i<8; ++i) {
+                        int tmp = abs(b[i+0]-EX->ub[src*4+0]);
+                        tmp += abs(b[i+1]-EX->ub[src*4+1]);
+                        tmp += abs(b[i+2]-EX->ub[src*4+2]);
+                        tmp += abs(b[i+3]-EX->ub[src*4+3]);
+                        GX->uw[i] = tmp;
+                    }
+                }
+                break;
 
             case 0x44:  /* PCLMULQDQ Gx, Ex, Ib */
                 nextop = F8;
@@ -827,6 +1119,65 @@ uintptr_t Run660F(x64emu_t *emu, rex_t rex, uintptr_t addr)
                 }
                 break;
 
+            case 0x60:  /* PCMPESTRM */
+                nextop = F8;
+                GETEX(1);
+                GETGX;
+                tmp8u = F8;
+                tmp32u = sse42_compare_string_explicit_len(emu, EX, R_EDX, GX, R_EAX, tmp8u);
+                if(tmp8u&0b1000000) {
+                    switch(tmp8u&1) {
+                        case 0: for(int i=0; i<16; ++i) GX->ub[i] = ((tmp32u>>i)&1)?0xff:0x00; break;
+                        case 1: for(int i=0; i<8; ++i) GX->uw[i] = ((tmp32u>>i)&1)?0xffff:0x0000; break;
+                    }
+                } else {
+                    GX->q[1] = GX->q[0] = 0;
+                    GX->uw[0] = tmp32u;
+                }
+                break;
+            case 0x61:  /* PCMPESTRI */
+                nextop = F8;
+                GETEX(1);
+                GETGX;
+                tmp8u = F8;
+                tmp32u = sse42_compare_string_explicit_len(emu, EX, R_EDX, GX, R_EAX, tmp8u);
+                if(!tmp32u)
+                    R_RCX = (tmp8u&1)?8:16;
+                else if(tmp8u&0b1000000)
+                    R_RCX = 31-__builtin_clz(tmp32u);
+                else
+                    R_RCX = __builtin_ffs(tmp32u) - 1;
+                break;
+            case 0x62:  /* PCMPISTRM */
+                nextop = F8;
+                GETEX(1);
+                GETGX;
+                tmp8u = F8;
+                tmp32u = sse42_compare_string_implicit_len(emu, EX, GX, tmp8u);
+                if(tmp8u&0b1000000) {
+                    switch(tmp8u&1) {
+                        case 0: for(int i=0; i<16; ++i) GX->ub[i] = ((tmp32u>>i)&1)?0xff:0x00; break;
+                        case 1: for(int i=0; i<8; ++i) GX->uw[i] = ((tmp32u>>i)&1)?0xffff:0x0000; break;
+                    }
+                } else {
+                    GX->q[1] = GX->q[0] = 0;
+                    GX->uw[0] = tmp32u;
+                }
+                break;
+            case 0x63:  /* PCMPISTRI */
+                nextop = F8;
+                GETEX(1);
+                GETGX;
+                tmp8u = F8;
+                tmp32u = sse42_compare_string_implicit_len(emu, EX, GX, tmp8u);
+                if(!tmp32u)
+                    R_RCX = (tmp8u&1)?8:16;
+                else if(tmp8u&0b1000000)
+                    R_RCX = 31-__builtin_clz(tmp32u);
+                else
+                    R_RCX = __builtin_ffs(tmp32u) - 1;
+                break;
+
             case 0xDF:      // AESKEYGENASSIST Gx, Ex, u8
                 nextop = F8;
                 GETEX(1);
@@ -852,7 +1203,7 @@ uintptr_t Run660F(x64emu_t *emu, rex_t rex, uintptr_t addr)
                 return 0;
         }
         break;
-        
+
     GOCOND(0x40
         , nextop = F8;
         CHECK_FLAGS(emu);
@@ -952,24 +1303,34 @@ uintptr_t Run660F(x64emu_t *emu, rex_t rex, uintptr_t addr)
         nextop = F8;
         GETEX(0);
         GETGX;
-        for(int i=0; i<4; ++i)
-            if(isnanf(EX->f[i]) || isinff(EX->f[i]) || EX->f[i]>0x7fffffff || EX->f[i]<-0x80000000)
-                GX->sd[i] = 0x80000000;
+        for(int i=0; i<4; ++i) {
+            if(isnanf(EX->f[i]))
+                tmp64s = INT32_MIN;
             else
                 switch(emu->mxcsr.f.MXCSR_RC) {
-                    case ROUND_Nearest:
-                        GX->sd[i] = nearbyintf(EX->f[i]);
+                    case ROUND_Nearest: {
+                        int round = fegetround();
+                        fesetround(FE_TONEAREST);
+                        tmp64s = nearbyintf(EX->f[i]);
+                        fesetround(round);
                         break;
+                    }
                     case ROUND_Down:
-                        GX->sd[i] = floorf(EX->f[i]);
+                        tmp64s = floorf(EX->f[i]);
                         break;
                     case ROUND_Up:
-                        GX->sd[i] = ceilf(EX->f[i]);
+                        tmp64s = ceilf(EX->f[i]);
                         break;
                     case ROUND_Chop:
-                        GX->sd[i] = EX->f[i];
+                        tmp64s = EX->f[i];
                         break;
                 }
+            if (tmp64s==(int32_t)tmp64s) {
+                GX->sd[i] = (int32_t)tmp64s;
+            } else {
+                GX->sd[i] = INT32_MIN;
+            }
+        }
         break;
     case 0x5C:                      /* SUBPD Gx, Ex */
         nextop = F8;
@@ -1018,7 +1379,7 @@ uintptr_t Run660F(x64emu_t *emu, rex_t rex, uintptr_t addr)
         if (isnan(GX->d[1]) || isnan(EX->d[1]) || isgreater(EX->d[1], GX->d[1]))
             GX->d[1] = EX->d[1];
         break;
-        
+
     case 0x60:  /* PUNPCKLBW Gx,Ex */
         nextop = F8;
         GETEX(0);
@@ -1028,7 +1389,7 @@ uintptr_t Run660F(x64emu_t *emu, rex_t rex, uintptr_t addr)
         if(GX==EX)
             for(int i=0; i<8; ++i)
                 GX->ub[2 * i + 1] = GX->ub[2 * i];
-        else 
+        else
             for(int i=0; i<8; ++i)
                 GX->ub[2 * i + 1] = EX->ub[i];
         break;
@@ -1263,7 +1624,7 @@ uintptr_t Run660F(x64emu_t *emu, rex_t rex, uintptr_t addr)
                 tmp8u = F8;
                 if(tmp8u>15)
                     {EX->q[0] = EX->q[1] = 0;}
-                else {
+                else if (tmp8u!=0) {
                     tmp8u*=8;
                     if (tmp8u < 64) {
                         EX->q[0] = (EX->q[0] >> tmp8u) | (EX->q[1] << (64 - tmp8u));
@@ -1271,7 +1632,7 @@ uintptr_t Run660F(x64emu_t *emu, rex_t rex, uintptr_t addr)
                     } else {
                         EX->q[0] = EX->q[1] >> (tmp8u - 64);
                         EX->q[1] = 0;
-                    }                    
+                    }
                 }
                 break;
             case 6:                 /* PSLLQ Ex, Ib */
@@ -1285,7 +1646,7 @@ uintptr_t Run660F(x64emu_t *emu, rex_t rex, uintptr_t addr)
                 tmp8u = F8;
                 if(tmp8u>15)
                     {EX->q[0] = EX->q[1] = 0;}
-                else {
+                else if (tmp8u!=0) {
                     tmp8u*=8;
                     if (tmp8u < 64) {
                         EX->q[1] = (EX->q[1] << tmp8u) | (EX->q[0] >> (64 - tmp8u));
@@ -1516,7 +1877,15 @@ uintptr_t Run660F(x64emu_t *emu, rex_t rex, uintptr_t addr)
         tmp32s >>= (rex.w?6:4);
         if(!MODREG)
         {
+            #ifdef TEST_INTERPRETER
+            test->memaddr=((test->memaddr)+(tmp32s<<(rex.w?3:1)));
+            if(rex.w)
+                *(uint64_t*)test->mem = *(uint64_t*)test->memaddr;
+            else
+                *(uint16_t*)test->mem = *(uint16_t*)test->memaddr;
+            #else
             EW=(reg64_t*)(((uintptr_t)(EW))+(tmp32s<<(rex.w?3:1)));
+            #endif
         }
         if(rex.w) {
             if(EW->q[0] & (1LL<<tmp8u)) {
@@ -1552,7 +1921,7 @@ uintptr_t Run660F(x64emu_t *emu, rex_t rex, uintptr_t addr)
             GW->word[0] = EW->word[0];
         break;
 
-    case 0xBA:                      
+    case 0xBA:
         nextop = F8;
         switch((nextop>>3)&7) {
             case 4:                 /* BT Ew,Ib */
@@ -1821,7 +2190,7 @@ uintptr_t Run660F(x64emu_t *emu, rex_t rex, uintptr_t addr)
         GETGX;
         if(EX->q[0]>15)
             {GX->q[0] = GX->q[1] = 0;}
-        else 
+        else
             {tmp8u=EX->ub[0]; for (int i=0; i<8; ++i) GX->uw[i] >>= tmp8u;}
         break;
     case 0xD2:  /* PSRLD Gx, Ex */
@@ -1830,7 +2199,7 @@ uintptr_t Run660F(x64emu_t *emu, rex_t rex, uintptr_t addr)
         GETGX;
         if(EX->q[0]>31)
             {GX->q[0] = GX->q[1] = 0;}
-        else 
+        else
             {tmp8u=EX->ub[0]; for (int i=0; i<4; ++i) GX->ud[i] >>= tmp8u;}
         break;
     case 0xD3:  /* PSRLQ Gx, Ex */
@@ -1839,7 +2208,7 @@ uintptr_t Run660F(x64emu_t *emu, rex_t rex, uintptr_t addr)
         GETGX;
         if(EX->q[0]>63)
             {GX->q[0] = GX->q[1] = 0;}
-        else 
+        else
             {tmp8u=EX->ub[0]; for (int i=0; i<2; ++i) GX->q[i] >>= tmp8u;}
         break;
     case 0xD4:  /* PADDQ Gx,Ex */
@@ -1954,7 +2323,7 @@ uintptr_t Run660F(x64emu_t *emu, rex_t rex, uintptr_t addr)
         GETEX(0);
         GETGX;
         tmp8u=(EX->q[0]>15)?15:EX->ub[0];
-        for (int i=0; i<8; ++i) 
+        for (int i=0; i<8; ++i)
             GX->sw[i] >>= tmp8u;
         break;
     case 0xE2:  /* PSRAD Gx, Ex */
@@ -2082,7 +2451,7 @@ uintptr_t Run660F(x64emu_t *emu, rex_t rex, uintptr_t addr)
         GETGX;
         if(EX->q[0]>15)
             {GX->q[0] = GX->q[1] = 0;}
-        else 
+        else
             {tmp8u=EX->ub[0]; for (int i=0; i<8; ++i) GX->uw[i] <<= tmp8u;}
         break;
     case 0xF2:  /* PSLLD Gx, Ex */
@@ -2091,7 +2460,7 @@ uintptr_t Run660F(x64emu_t *emu, rex_t rex, uintptr_t addr)
         GETGX;
         if(EX->q[0]>31)
             {GX->q[0] = GX->q[1] = 0;}
-        else 
+        else
             {tmp8u=EX->ub[0]; for (int i=0; i<4; ++i) GX->ud[i] <<= tmp8u;}
         break;
     case 0xF3:  /* PSLLQ Gx, Ex */
@@ -2100,7 +2469,7 @@ uintptr_t Run660F(x64emu_t *emu, rex_t rex, uintptr_t addr)
         GETGX;
         if(EX->q[0]>63)
             {GX->q[0] = GX->q[1] = 0;}
-        else 
+        else
             {tmp8u=EX->ub[0]; for (int i=0; i<2; ++i) GX->q[i] <<= tmp8u;}
         break;
     case 0xF4:  /* PMULUDQ Gx,Ex */
@@ -2193,9 +2562,9 @@ uintptr_t Run660F(x64emu_t *emu, rex_t rex, uintptr_t addr)
         GX->sd[2] += EX->sd[2];
         GX->sd[3] += EX->sd[3];
         break;
-    
+
     default:
         return 0;
     }
     return addr;
-}
\ No newline at end of file
+}
