@@ -26,7 +26,11 @@
 
 #include "modrm.h"
 
+#ifdef TEST_INTERPRETER
+uintptr_t TestF0(x64test_t *test, rex_t rex, uintptr_t addr)
+#else
 uintptr_t RunF0(x64emu_t *emu, rex_t rex, uintptr_t addr)
+#endif
 {
     uint8_t opcode;
     uint8_t nextop;
@@ -36,17 +40,26 @@ uintptr_t RunF0(x64emu_t *emu, rex_t rex, uintptr_t addr)
     int64_t tmp64s;
     uint64_t tmp64u, tmp64u2;
     reg64_t *oped, *opgd;
+    #ifdef USE_CAS
+    uint64_t tmpcas;
+    #endif
+    #ifdef TEST_INTERPRETER
+    x64emu_t*emu = test->emu;
+    #endif
 
     opcode = F8;
+    while(opcode==0x2E) // ignoring CS: prefix
+        opcode = F8;
     // REX prefix before the F0 are ignored
     rex.rex = 0;
-    while(opcode>=0x40 && opcode<=0x4f) {
-        rex.rex = opcode;
-        opcode = F8;
-    }
+    if(!rex.is32bits)
+        while(opcode>=0x40 && opcode<=0x4f) {
+            rex.rex = opcode;
+            opcode = F8;
+        }
 
     switch(opcode) {
-#ifdef DYNAREC
+#if defined(DYNAREC) && !defined(TEST_INTERPRETER)
         #define GO(B, OP)                                           \
         case B+0:                                                   \
             nextop = F8;                                            \
@@ -105,15 +118,15 @@ uintptr_t RunF0(x64emu_t *emu, rex_t rex, uintptr_t addr)
             nextop = F8;                                            \
             GETEB(0);                                               \
             GETGB;                                                  \
-            pthread_mutex_lock(&emu->context->mutex_lock);          \
+            pthread_mutex_lock(&my_context->mutex_lock);          \
             EB->byte[0] = OP##8(emu, EB->byte[0], GB);              \
-            pthread_mutex_unlock(&emu->context->mutex_lock);        \
+            pthread_mutex_unlock(&my_context->mutex_lock);        \
             break;                                                  \
         case B+1:                                                   \
             nextop = F8;                                            \
             GETED(0);                                               \
             GETGD;                                                  \
-            pthread_mutex_lock(&emu->context->mutex_lock);          \
+            pthread_mutex_lock(&my_context->mutex_lock);          \
             if(rex.w)                                               \
                 ED->q[0] = OP##64(emu, ED->q[0], GD->q[0]);         \
             else                                                    \
@@ -121,39 +134,39 @@ uintptr_t RunF0(x64emu_t *emu, rex_t rex, uintptr_t addr)
                     ED->q[0] = OP##32(emu, ED->dword[0], GD->dword[0]);     \
                 else                                                        \
                     ED->dword[0] = OP##32(emu, ED->dword[0], GD->dword[0]); \
-            pthread_mutex_unlock(&emu->context->mutex_lock);        \
+            pthread_mutex_unlock(&my_context->mutex_lock);        \
             break;                                                  \
         case B+2:                                                   \
             nextop = F8;                                            \
             GETEB(0);                                               \
             GETGB;                                                  \
-            pthread_mutex_lock(&emu->context->mutex_lock);          \
+            pthread_mutex_lock(&my_context->mutex_lock);          \
             GB = OP##8(emu, GB, EB->byte[0]);                       \
-            pthread_mutex_unlock(&emu->context->mutex_lock);        \
+            pthread_mutex_unlock(&my_context->mutex_lock);        \
             break;                                                  \
         case B+3:                                                   \
             nextop = F8;                                            \
             GETED(0);                                               \
             GETGD;                                                  \
-            pthread_mutex_lock(&emu->context->mutex_lock);          \
+            pthread_mutex_lock(&my_context->mutex_lock);          \
             if(rex.w)                                               \
                 GD->q[0] = OP##64(emu, GD->q[0], ED->q[0]);         \
             else                                                    \
                 GD->q[0] = OP##32(emu, GD->dword[0], ED->dword[0]); \
-            pthread_mutex_unlock(&emu->context->mutex_lock);        \
+            pthread_mutex_unlock(&my_context->mutex_lock);        \
             break;                                                  \
         case B+4:                                                   \
-            pthread_mutex_lock(&emu->context->mutex_lock);          \
+            pthread_mutex_lock(&my_context->mutex_lock);          \
             R_AL = OP##8(emu, R_AL, F8);                            \
-            pthread_mutex_unlock(&emu->context->mutex_lock);        \
+            pthread_mutex_unlock(&my_context->mutex_lock);        \
             break;                                                  \
         case B+5:                                                   \
-            pthread_mutex_lock(&emu->context->mutex_lock);          \
+            pthread_mutex_lock(&my_context->mutex_lock);          \
             if(rex.w)                                               \
                 R_RAX = OP##64(emu, R_RAX, F32S64);                 \
             else                                                    \
                 R_RAX = OP##32(emu, R_EAX, F32);                    \
-            pthread_mutex_unlock(&emu->context->mutex_lock);        \
+            pthread_mutex_unlock(&my_context->mutex_lock);        \
             break;
 #endif
         GO(0x00, add)                   /* ADD 0x00 -> 0x05 */
@@ -178,9 +191,17 @@ uintptr_t RunF0(x64emu_t *emu, rex_t rex, uintptr_t addr)
                 tmp64s >>= (rex.w?6:5);
                 if(!MODREG)
                 {
+                    #ifdef TEST_INTERPRETER
+                    test->memaddr=((test->memaddr)+(tmp32s<<(rex.w?3:2)));
+                    if(rex.w)
+                        *(uint64_t*)test->mem = *(uint64_t*)test->memaddr;
+                    else
+                        *(uint32_t*)test->mem = *(uint32_t*)test->memaddr;
+                    #else
                     ED=(reg64_t*)(((uintptr_t)(ED))+(tmp64s<<(rex.w?3:2)));
+                    #endif
                 }
-#ifdef DYNAREC
+#if defined(DYNAREC) && !defined(TEST_INTERPRETER)
                 if(rex.w) {
                     tmp8u&=63;
                     if(MODREG) {
@@ -226,7 +247,7 @@ uintptr_t RunF0(x64emu_t *emu, rex_t rex, uintptr_t addr)
                         } while(tmp32s);
                 }
 #else
-                pthread_mutex_lock(&emu->context->mutex_lock);
+                pthread_mutex_lock(&my_context->mutex_lock);
                 if(rex.w) {
                     tmp8u&=63;
                     if(ED->q[0] & (1LL<<tmp8u))
@@ -246,7 +267,7 @@ uintptr_t RunF0(x64emu_t *emu, rex_t rex, uintptr_t addr)
                     if(MODREG)
                         ED->dword[1] = 0;
                 }
-                pthread_mutex_unlock(&emu->context->mutex_lock);
+                pthread_mutex_unlock(&my_context->mutex_lock);
 #endif
                 break;
 
@@ -255,7 +276,7 @@ uintptr_t RunF0(x64emu_t *emu, rex_t rex, uintptr_t addr)
                     nextop = F8;
                     GETGB;
                     GETEB(0);
-#ifdef DYNAREC
+#if defined(DYNAREC) && !defined(TEST_INTERPRETER)
                     do {
                         tmp8u = native_lock_read_b(EB);
                         cmp8(emu, R_AL, tmp8u);
@@ -267,21 +288,21 @@ uintptr_t RunF0(x64emu_t *emu, rex_t rex, uintptr_t addr)
                         }
                     } while(tmp32s);
 #else
-                    pthread_mutex_lock(&emu->context->mutex_lock);
+                    pthread_mutex_lock(&my_context->mutex_lock);
                     cmp8(emu, R_AL, EB->byte[0]);
                     if(ACCESS_FLAG(F_ZF)) {
                         EB->byte[0] = GB;
                     } else {
                         R_AL = EB->byte[0];
                     }
-                    pthread_mutex_unlock(&emu->context->mutex_lock);
+                    pthread_mutex_unlock(&my_context->mutex_lock);
 #endif
                     break;
                 case 0xB1:                      /* CMPXCHG Ed,Gd */
                     nextop = F8;
                     GETED(0);
                     GETGD;
-#ifdef DYNAREC
+#if defined(DYNAREC) && !defined(TEST_INTERPRETER)
                     if(rex.w)
                         if(((uintptr_t)ED)&7) {
                             do {
@@ -309,22 +330,37 @@ uintptr_t RunF0(x64emu_t *emu, rex_t rex, uintptr_t addr)
                                 }
                             } while(tmp32s);
                     else {
-                        do {
-                            tmp32u = native_lock_read_d(ED);
-                            cmp32(emu, R_EAX, tmp32u);
-                            if(ACCESS_FLAG(F_ZF)) {
-                                tmp32s = native_lock_write_d(ED, GD->dword[0]);
-                            } else {
-                                R_EAX = tmp32u;
-                                tmp32s = 0;
-                            }
-                        } while(tmp32s);
+                        if(((uintptr_t)ED)&3) {
+                            do {
+                                tmp32u = ED->q[0] & ~0xffLL;
+                                tmp32u |= native_lock_read_b(ED);
+                                cmp64(emu, R_RAX, tmp32u);
+                                if(ACCESS_FLAG(F_ZF)) {
+                                    tmp32s = native_lock_write_b(ED, GD->dword[0]&0xff);
+                                    if(!tmp32s)
+                                        ED->dword[0] = GD->dword[0];
+                                } else {
+                                    R_EAX = tmp32u;
+                                    tmp32s = 0;
+                                }
+                            } while(tmp32s);
+                        } else
+                            do {
+                                tmp32u = native_lock_read_d(ED);
+                                cmp32(emu, R_EAX, tmp32u);
+                                if(ACCESS_FLAG(F_ZF)) {
+                                    tmp32s = native_lock_write_d(ED, GD->dword[0]);
+                                } else {
+                                    R_EAX = tmp32u;
+                                    tmp32s = 0;
+                                }
+                            } while(tmp32s);
                         emu->regs[_AX].dword[1] = 0;
                         if(MODREG)
                             ED->dword[1] = 0;
                     }
 #else
-                    pthread_mutex_lock(&emu->context->mutex_lock);
+                    pthread_mutex_lock(&my_context->mutex_lock);
                     if(rex.w) {
                         cmp64(emu, R_RAX, ED->q[0]);
                         if(ACCESS_FLAG(F_ZF)) {
@@ -343,7 +379,7 @@ uintptr_t RunF0(x64emu_t *emu, rex_t rex, uintptr_t addr)
                         if(MODREG)
                             ED->dword[1] = 0;
                     }
-                    pthread_mutex_unlock(&emu->context->mutex_lock);
+                    pthread_mutex_unlock(&my_context->mutex_lock);
 #endif
                     break;
 
@@ -357,10 +393,18 @@ uintptr_t RunF0(x64emu_t *emu, rex_t rex, uintptr_t addr)
                     tmp64s >>= (rex.w?6:5);
                     if(!MODREG)
                     {
+                        #ifdef TEST_INTERPRETER
+                        test->memaddr=((test->memaddr)+(tmp32s<<(rex.w?3:2)));
+                        if(rex.w)
+                            *(uint64_t*)test->mem = *(uint64_t*)test->memaddr;
+                        else
+                            *(uint32_t*)test->mem = *(uint32_t*)test->memaddr;
+                        #else
                         ED=(reg64_t*)(((uintptr_t)(ED))+(tmp64s<<(rex.w?3:2)));
+                        #endif
                     }
                     tmp8u&=rex.w?63:31;
-#ifdef DYNAREC
+#if defined(DYNAREC) && !defined(TEST_INTERPRETER)
                     if(rex.w)
                         do {
                             tmp64u = native_lock_read_dd(ED);
@@ -389,7 +433,7 @@ uintptr_t RunF0(x64emu_t *emu, rex_t rex, uintptr_t addr)
                             ED->dword[1] = 0;
                     }
 #else
-                    pthread_mutex_lock(&emu->context->mutex_lock);
+                    pthread_mutex_lock(&my_context->mutex_lock);
                     if(rex.w) {
                         if(ED->q[0] & (1<<tmp8u)) {
                             SET_FLAG(F_CF);
@@ -405,7 +449,7 @@ uintptr_t RunF0(x64emu_t *emu, rex_t rex, uintptr_t addr)
                         if(MODREG)
                             ED->dword[1] = 0;
                     }
-                    pthread_mutex_unlock(&emu->context->mutex_lock);
+                    pthread_mutex_unlock(&my_context->mutex_lock);
 #endif
                     break;
 
@@ -434,7 +478,7 @@ uintptr_t RunF0(x64emu_t *emu, rex_t rex, uintptr_t addr)
                                 CHECK_FLAGS(emu);
                                 GETED(1);
                                 tmp8u = F8;
-#ifdef DYNAREC
+#if defined(DYNAREC) && !defined(TEST_INTERPRETER)
                                 if(rex.w) {
                                     tmp8u&=63;
                                     do {
@@ -463,7 +507,7 @@ uintptr_t RunF0(x64emu_t *emu, rex_t rex, uintptr_t addr)
                                     } while(tmp32s);
                                 }
 #else
-                                pthread_mutex_lock(&emu->context->mutex_lock);
+                                pthread_mutex_lock(&my_context->mutex_lock);
                                 if(rex.w) {
                                     tmp8u&=63;
                                     if(ED->q[0] & (1LL<<tmp8u)) {
@@ -481,14 +525,14 @@ uintptr_t RunF0(x64emu_t *emu, rex_t rex, uintptr_t addr)
                                         CLEAR_FLAG(F_CF);
                                     }
                                 }
-                                pthread_mutex_unlock(&emu->context->mutex_lock);
+                                pthread_mutex_unlock(&my_context->mutex_lock);
 #endif
                                 break;
                             case 6:             /* BTR Ed, Ib */
                                 CHECK_FLAGS(emu);
                                 GETED(1);
                                 tmp8u = F8;
-#ifdef DYNAREC
+#if defined(DYNAREC) && !defined(TEST_INTERPRETER)
                                 if(rex.w) {
                                     do {
                                         tmp8u&=63;
@@ -517,7 +561,7 @@ uintptr_t RunF0(x64emu_t *emu, rex_t rex, uintptr_t addr)
                                     } while(tmp32s);
                                 }
 #else
-                                pthread_mutex_lock(&emu->context->mutex_lock);
+                                pthread_mutex_lock(&my_context->mutex_lock);
                                 if(rex.w) {
                                     tmp8u&=63;
                                     if(ED->q[0] & (1LL<<tmp8u)) {
@@ -533,14 +577,14 @@ uintptr_t RunF0(x64emu_t *emu, rex_t rex, uintptr_t addr)
                                     } else
                                         CLEAR_FLAG(F_CF);
                                 }
-                                pthread_mutex_unlock(&emu->context->mutex_lock);
+                                pthread_mutex_unlock(&my_context->mutex_lock);
 #endif
                                 break;
                             case 7:             /* BTC Ed, Ib */
                                 CHECK_FLAGS(emu);
                                 GETED(1);
                                 tmp8u = F8;
-#ifdef DYNAREC
+#if defined(DYNAREC) && !defined(TEST_INTERPRETER)
                                 if(rex.w) {
                                     tmp8u&=63;
                                     do {
@@ -565,7 +609,7 @@ uintptr_t RunF0(x64emu_t *emu, rex_t rex, uintptr_t addr)
                                     } while(tmp32s);
                                 }
 #else
-                                pthread_mutex_lock(&emu->context->mutex_lock);
+                                pthread_mutex_lock(&my_context->mutex_lock);
                                 if(rex.w) {
                                     tmp8u&=63;
                                     if(ED->q[0] & (1LL<<tmp8u))
@@ -581,7 +625,7 @@ uintptr_t RunF0(x64emu_t *emu, rex_t rex, uintptr_t addr)
                                         CLEAR_FLAG(F_CF);
                                     ED->dword[0] ^= (1<<tmp8u);
                                 }
-                                pthread_mutex_unlock(&emu->context->mutex_lock);
+                                pthread_mutex_unlock(&my_context->mutex_lock);
 #endif
                                 break;
 
@@ -594,25 +638,25 @@ uintptr_t RunF0(x64emu_t *emu, rex_t rex, uintptr_t addr)
                     nextop = F8;
                     GETEB(0);
                     GETGB;
-#ifdef DYNAREC
+#if defined(DYNAREC) && !defined(TEST_INTERPRETER)
                     do {
                         tmp8u = native_lock_read_b(EB);
                         tmp8u2 = add8(emu, tmp8u, GB);
                     } while(native_lock_write_b(EB, tmp8u2));
                     GB = tmp8u;
 #else
-                    pthread_mutex_lock(&emu->context->mutex_lock);
+                    pthread_mutex_lock(&my_context->mutex_lock);
                     tmp8u = add8(emu, EB->byte[0], GB);
                     GB = EB->byte[0];
                     EB->byte[0] = tmp8u;
-                    pthread_mutex_unlock(&emu->context->mutex_lock);
+                    pthread_mutex_unlock(&my_context->mutex_lock);
 #endif
                     break;
                 case 0xC1:                      /* XADD Gd,Ed */
                     nextop = F8;
                     GETED(0);
                     GETGD;
-#ifdef DYNAREC
+#if defined(DYNAREC) && !defined(TEST_INTERPRETER)
                     if(rex.w) {
                         do {
                             tmp64u = native_lock_read_dd(ED);
@@ -638,31 +682,33 @@ uintptr_t RunF0(x64emu_t *emu, rex_t rex, uintptr_t addr)
                             ED->dword[1] = 0;
                     }
 #else
-                    pthread_mutex_lock(&emu->context->mutex_lock);
+                    pthread_mutex_lock(&my_context->mutex_lock);
                     if(rex.w) {
-                        tmp64u = add64(emu, ED->q[0], GD->q[0]);
-                        GD->q[0] = ED->q[0];
-                        ED->q[0] = tmp64u;
+                        tmp64u = ED->q[0];
+                        tmp64u2 = add64(emu, tmp64u, GD->q[0]);
+                        GD->q[0] = tmp64u;
+                        ED->q[0] = tmp64u2;
                     } else {
-                        tmp32u = add32(emu, ED->dword[0], GD->dword[0]);
-                        GD->q[0] = ED->dword[0];
+                        tmp32u = ED->dword[0];
+                        tmp32u2 = add32(emu, tmp32u, GD->dword[0]);
+                        GD->q[0] = tmp32u;
                         if(MODREG)
-                            ED->q[0] = tmp32u;
+                            ED->q[0] = tmp32u2;
                         else
-                            ED->dword[0] = tmp32u;
+                            ED->dword[0] = tmp32u2;
                     }
-                    pthread_mutex_unlock(&emu->context->mutex_lock);
+                    pthread_mutex_unlock(&my_context->mutex_lock);
 #endif
                     break;
 
                 case 0xC7:                      /* CMPXCHG8B Gq */
                     nextop = F8;
-                    GETED(0);
+                    GETE8xw(0);
                     switch((nextop>>3)&7) {
                         case 1:
                             CHECK_FLAGS(emu);
                             GETGD;
-#ifdef DYNAREC
+#if defined(DYNAREC) && !defined(TEST_INTERPRETER)
                             if(rex.w)
                                 do {
                                     native_lock_read_dq(&tmp64u, &tmp64u2, ED);
@@ -690,8 +736,11 @@ uintptr_t RunF0(x64emu_t *emu, rex_t rex, uintptr_t addr)
                                     }
                                 } while(tmp32s);
 #else
-                            pthread_mutex_lock(&emu->context->mutex_lock);
+                            pthread_mutex_lock(&my_context->mutex_lock);
                             if(rex.w) {
+                                #ifdef TEST_INTERPRETER
+                                test->memsize = 16;
+                                #endif
                                 tmp64u = ED->q[0];
                                 tmp64u2= ED->q[1];
                                 if(R_RAX == tmp64u && R_RDX == tmp64u2) {
@@ -704,6 +753,9 @@ uintptr_t RunF0(x64emu_t *emu, rex_t rex, uintptr_t addr)
                                     R_RDX = tmp64u2;
                                 }
                             } else {
+                                #ifdef TEST_INTERPRETER
+                                test->memsize = 8;
+                                #endif
                                 tmp32u = ED->dword[0];
                                 tmp32u2= ED->dword[1];
                                 if(R_EAX == tmp32u && R_EDX == tmp32u2) {
@@ -716,7 +768,7 @@ uintptr_t RunF0(x64emu_t *emu, rex_t rex, uintptr_t addr)
                                     R_RDX = tmp32u2;
                                 }
                             }
-                            pthread_mutex_unlock(&emu->context->mutex_lock);
+                            pthread_mutex_unlock(&my_context->mutex_lock);
 #endif
                             break;
                         default:
@@ -730,13 +782,17 @@ uintptr_t RunF0(x64emu_t *emu, rex_t rex, uintptr_t addr)
             break;
 
         case 0x66:
+            #ifdef TEST_INTERPRETER
+            return Test66F0(test, rex, addr);
+            #else
             return Run66F0(emu, rex, addr);   // more opcode F0 66 and 66 F0 is the same
+            #endif
 
         case 0x80:                      /* GRP Eb,Ib */
             nextop = F8;
             GETEB(1);
             tmp8u = F8;
-#ifdef DYNAREC
+#if defined(DYNAREC) && !defined(TEST_INTERPRETER)
             switch((nextop>>3)&7) {
                 case 0: do { tmp8u2 = native_lock_read_b(EB); tmp8u2 = add8(emu, tmp8u2, tmp8u);} while(native_lock_write_b(EB, tmp8u2)); break;
                 case 1: do { tmp8u2 = native_lock_read_b(EB); tmp8u2 =  or8(emu, tmp8u2, tmp8u);} while(native_lock_write_b(EB, tmp8u2)); break;
@@ -748,7 +804,7 @@ uintptr_t RunF0(x64emu_t *emu, rex_t rex, uintptr_t addr)
                 case 7:               cmp8(emu, EB->byte[0], tmp8u); break;
             }
 #else
-            pthread_mutex_lock(&emu->context->mutex_lock);
+            pthread_mutex_lock(&my_context->mutex_lock);
             switch((nextop>>3)&7) {
                 case 0: EB->byte[0] = add8(emu, EB->byte[0], tmp8u); break;
                 case 1: EB->byte[0] =  or8(emu, EB->byte[0], tmp8u); break;
@@ -759,7 +815,7 @@ uintptr_t RunF0(x64emu_t *emu, rex_t rex, uintptr_t addr)
                 case 6: EB->byte[0] = xor8(emu, EB->byte[0], tmp8u); break;
                 case 7:               cmp8(emu, EB->byte[0], tmp8u); break;
             }
-            pthread_mutex_unlock(&emu->context->mutex_lock);
+            pthread_mutex_unlock(&my_context->mutex_lock);
 #endif
             break;
         case 0x81:              /* GRP Ed,Id */
@@ -771,7 +827,7 @@ uintptr_t RunF0(x64emu_t *emu, rex_t rex, uintptr_t addr)
                 tmp64u = (uint64_t)tmp64s;
             } else
                 tmp64u = F32S64;
-#ifdef DYNAREC
+#if defined(DYNAREC) && !defined(TEST_INTERPRETER)
             if(rex.w) {
                 switch((nextop>>3)&7) {
                     case 0: do { tmp64u2 = native_lock_read_dd(ED); tmp64u2 = add64(emu, tmp64u2, tmp64u);} while(native_lock_write_dd(ED, tmp64u2)); break;
@@ -808,7 +864,7 @@ uintptr_t RunF0(x64emu_t *emu, rex_t rex, uintptr_t addr)
                     }
             }
 #else
-            pthread_mutex_lock(&emu->context->mutex_lock);
+            pthread_mutex_lock(&my_context->mutex_lock);
             if(rex.w) {
                 switch((nextop>>3)&7) {
                     case 0: ED->q[0] = add64(emu, ED->q[0], tmp64u); break;
@@ -844,13 +900,13 @@ uintptr_t RunF0(x64emu_t *emu, rex_t rex, uintptr_t addr)
                         case 7:                cmp32(emu, ED->dword[0], tmp64u); break;
                     }
             }
-            pthread_mutex_unlock(&emu->context->mutex_lock);
+            pthread_mutex_unlock(&my_context->mutex_lock);
 #endif
             break;
 
         case 0x86:                      /* XCHG Eb,Gb */
             nextop = F8;
-#ifdef DYNAREC
+#if defined(DYNAREC) && !defined(TEST_INTERPRETER)
             GETEB(0);
             GETGB;
             if(MODREG) { // reg / reg: no lock
@@ -867,17 +923,17 @@ uintptr_t RunF0(x64emu_t *emu, rex_t rex, uintptr_t addr)
             GETEB(0);
             GETGB;
             if(!MODREG)
-                pthread_mutex_lock(&emu->context->mutex_lock); // XCHG always LOCK (but when accessing memory only)
+                pthread_mutex_lock(&my_context->mutex_lock); // XCHG always LOCK (but when accessing memory only)
             tmp8u = GB;
             GB = EB->byte[0];
             EB->byte[0] = tmp8u;
             if(!MODREG)
-                pthread_mutex_unlock(&emu->context->mutex_lock);
+                pthread_mutex_unlock(&my_context->mutex_lock);
 #endif                
             break;
         case 0x87:                      /* XCHG Ed,Gd */
             nextop = F8;
-#ifdef DYNAREC
+#if defined(DYNAREC) && !defined(TEST_INTERPRETER)
             GETED(0);
             GETGD;
             if(MODREG) {
@@ -892,7 +948,18 @@ uintptr_t RunF0(x64emu_t *emu, rex_t rex, uintptr_t addr)
                 }
             } else {
                 if(rex.w) {
-                    GD->q[0] = native_lock_xchg(ED, GD->q[0]);
+                    if((uintptr_t)ED&7) {
+                        // unaligned
+                        do {
+                            tmp64u = ED->q[0] & 0xffffffffffffff00LL;
+                            tmp64u |= native_lock_read_b(ED);
+                            
+                        } while(native_lock_write_b(ED, GD->byte[0]));
+                        ED->q[0] = GD->q[0];
+                        GD->q[0] = tmp64u;
+                    } else {
+                        GD->q[0] = native_lock_xchg_dd(ED, GD->q[0]);
+                    }
                 } else {
                     GD->dword[0] = native_lock_xchg_d(ED, GD->dword[0]);
                 }
@@ -900,7 +967,7 @@ uintptr_t RunF0(x64emu_t *emu, rex_t rex, uintptr_t addr)
 #else
             GETED(0);
             GETGD;
-            pthread_mutex_lock(&emu->context->mutex_lock);
+            pthread_mutex_lock(&my_context->mutex_lock);
             if(rex.w) {
                 tmp64u = GD->q[0];
                 GD->q[0] = ED->q[0];
@@ -913,7 +980,7 @@ uintptr_t RunF0(x64emu_t *emu, rex_t rex, uintptr_t addr)
                 else
                     ED->dword[0] = tmp32u;
             }
-            pthread_mutex_unlock(&emu->context->mutex_lock);
+            pthread_mutex_unlock(&my_context->mutex_lock);
 #endif
             break;            
 
@@ -923,15 +990,15 @@ uintptr_t RunF0(x64emu_t *emu, rex_t rex, uintptr_t addr)
             GETEB((tmp8u<2)?1:0);
             switch(tmp8u) {
                 case 2:                 /* NOT Eb */
-#ifdef DYNAREC
+#if defined(DYNAREC) && !defined(TEST_INTERPRETER)
                     do {
                         tmp8u2 = native_lock_read_b(EB); 
                         tmp8u2 = not8(emu, tmp8u2);
                     } while(native_lock_write_b(EB, tmp8u2));
 #else
-                    pthread_mutex_lock(&emu->context->mutex_lock);
+                    pthread_mutex_lock(&my_context->mutex_lock);
                     EB->byte[0] = not8(emu, EB->byte[0]);
-                    pthread_mutex_unlock(&emu->context->mutex_lock);
+                    pthread_mutex_unlock(&my_context->mutex_lock);
 #endif
                     break;
                 default:
@@ -944,25 +1011,25 @@ uintptr_t RunF0(x64emu_t *emu, rex_t rex, uintptr_t addr)
             GETED(0);
             switch((nextop>>3)&7) {
                 case 0:                 /* INC Eb */
-#ifdef DYNAREC
+#if defined(DYNAREC) && !defined(TEST_INTERPRETER)
                     do {
                         tmp8u = native_lock_read_b(ED);
                     } while(native_lock_write_b(ED, inc8(emu, tmp8u)));
 #else
-                    pthread_mutex_lock(&emu->context->mutex_lock);
+                    pthread_mutex_lock(&my_context->mutex_lock);
                     ED->byte[0] = inc8(emu, ED->byte[0]);
-                    pthread_mutex_unlock(&emu->context->mutex_lock);
+                    pthread_mutex_unlock(&my_context->mutex_lock);
 #endif
                     break;
                 case 1:                 /* DEC Ed */
-#ifdef DYNAREC
+#if defined(DYNAREC) && !defined(TEST_INTERPRETER)
                     do {
                         tmp8u = native_lock_read_b(ED);
                     } while(native_lock_write_b(ED, dec8(emu, tmp8u)));
 #else
-                    pthread_mutex_lock(&emu->context->mutex_lock);
+                    pthread_mutex_lock(&my_context->mutex_lock);
                     ED->byte[0] = dec8(emu, ED->byte[0]);
-                    pthread_mutex_unlock(&emu->context->mutex_lock);
+                    pthread_mutex_unlock(&my_context->mutex_lock);
 #endif
                     break;
                 default:
@@ -977,7 +1044,7 @@ uintptr_t RunF0(x64emu_t *emu, rex_t rex, uintptr_t addr)
             GETED(0);
             switch((nextop>>3)&7) {
                 case 0:                 /* INC Ed */
-#ifdef DYNAREC
+#if defined(DYNAREC) && !defined(TEST_INTERPRETER)
                     if(rex.w)
                         if(((uintptr_t)ED)&7) {
                             // unaligned
@@ -1010,7 +1077,7 @@ uintptr_t RunF0(x64emu_t *emu, rex_t rex, uintptr_t addr)
                         if(MODREG) ED->dword[1] = 0;
                     }
 #else
-                    pthread_mutex_lock(&emu->context->mutex_lock);
+                    pthread_mutex_lock(&my_context->mutex_lock);
                     if(rex.w) {
                         ED->q[0] = inc64(emu, ED->q[0]);
                     } else {
@@ -1019,11 +1086,11 @@ uintptr_t RunF0(x64emu_t *emu, rex_t rex, uintptr_t addr)
                         else
                             ED->dword[0] = inc32(emu, ED->dword[0]);
                     }
-                    pthread_mutex_unlock(&emu->context->mutex_lock);
+                    pthread_mutex_unlock(&my_context->mutex_lock);
 #endif
                     break;
                 case 1:                 /* DEC Ed */
-#ifdef DYNAREC
+#if defined(DYNAREC) && !defined(TEST_INTERPRETER)
                     if(rex.w)
                         if(((uintptr_t)ED)&7) {
                             // unaligned
@@ -1045,7 +1112,7 @@ uintptr_t RunF0(x64emu_t *emu, rex_t rex, uintptr_t addr)
                         if(MODREG) ED->dword[1] = 0;
                     }
 #else
-                    pthread_mutex_lock(&emu->context->mutex_lock);
+                    pthread_mutex_lock(&my_context->mutex_lock);
                     if(rex.w) {
                         ED->q[0] = dec64(emu, ED->q[0]);
                     } else {
@@ -1054,7 +1121,7 @@ uintptr_t RunF0(x64emu_t *emu, rex_t rex, uintptr_t addr)
                         else
                             ED->dword[0] = dec32(emu, ED->dword[0]);
                     }
-                    pthread_mutex_unlock(&emu->context->mutex_lock);
+                    pthread_mutex_unlock(&my_context->mutex_lock);
 #endif
                     break;
                 default:
