@@ -38,7 +38,7 @@ GO(3)
 static uintptr_t my_DBusFreeFunction_fct_##A = 0;               \
 static void my_DBusFreeFunction_##A(void* p)                    \
 {                                                               \
-    RunFunction(my_context, my_DBusFreeFunction_fct_##A, 1, p); \
+    RunFunctionFmt(my_DBusFreeFunction_fct_##A, "p", p); \
 }
 SUPER()
 #undef GO
@@ -60,7 +60,7 @@ static void* find_DBusFreeFunction_Fct(void* fct)
 static uintptr_t my_DBusHandleMessageFunction_fct_##A = 0;                              \
 static int my_DBusHandleMessageFunction_##A(void* a, void* b, void* c)                  \
 {                                                                                       \
-    return RunFunction(my_context, my_DBusHandleMessageFunction_fct_##A, 3, a, b, c);   \
+    return RunFunctionFmt(my_DBusHandleMessageFunction_fct_##A, "ppp", a, b, c);   \
 }
 SUPER()
 #undef GO
@@ -82,7 +82,7 @@ static void* find_DBusHandleMessageFunction_Fct(void* fct)
 static uintptr_t my_DBusAddTimeoutFunction_fct_##A = 0;                         \
 static int my_DBusAddTimeoutFunction_##A(void* a, void* b)                      \
 {                                                                               \
-    return RunFunction(my_context, my_DBusAddTimeoutFunction_fct_##A, 2, a, b); \
+    return RunFunctionFmt(my_DBusAddTimeoutFunction_fct_##A, "pp", a, b); \
 }
 SUPER()
 #undef GO
@@ -104,7 +104,7 @@ static void* find_DBusAddTimeoutFunction_Fct(void* fct)
 static uintptr_t my_DBusRemoveTimeoutFunction_fct_##A = 0;                  \
 static void my_DBusRemoveTimeoutFunction_##A(void* a, void* b)              \
 {                                                                           \
-    RunFunction(my_context, my_DBusRemoveTimeoutFunction_fct_##A, 2, a, b); \
+    RunFunctionFmt(my_DBusRemoveTimeoutFunction_fct_##A, "pp", a, b); \
 }
 SUPER()
 #undef GO
@@ -126,7 +126,7 @@ static void* find_DBusRemoveTimeoutFunction_Fct(void* fct)
 static uintptr_t my_DBusTimeoutToggledFunction_fct_##A = 0;                     \
 static void my_DBusTimeoutToggledFunction_##A(void* a, void* b)                 \
 {                                                                               \
-    RunFunction(my_context, my_DBusTimeoutToggledFunction_fct_##A, 2, a, b);    \
+    RunFunctionFmt(my_DBusTimeoutToggledFunction_fct_##A, "pp", a, b);    \
 }
 SUPER()
 #undef GO
@@ -148,7 +148,7 @@ static void* find_DBusTimeoutToggledFunction_Fct(void* fct)
 static uintptr_t my_DBusWakeupMainFunction_fct_##A = 0;                 \
 static void my_DBusWakeupMainFunction_##A(void* a)                      \
 {                                                                       \
-    RunFunction(my_context, my_DBusWakeupMainFunction_fct_##A, 1, a);   \
+    RunFunctionFmt(my_DBusWakeupMainFunction_fct_##A, "p", a);   \
 }
 SUPER()
 #undef GO
@@ -171,7 +171,7 @@ static void* find_DBusWakeupMainFunction_Fct(void* fct)
 static uintptr_t my_DBusPendingCallNotifyFunction_fct_##A = 0;   \
 static void my_DBusPendingCallNotifyFunction_##A(void* pending, void* data)     \
 {                                       \
-    RunFunction(my_context, my_DBusPendingCallNotifyFunction_fct_##A, 2, pending, data);\
+    RunFunctionFmt(my_DBusPendingCallNotifyFunction_fct_##A, "pp", pending, data);\
 }
 SUPER()
 #undef GO
@@ -194,7 +194,7 @@ static void* findDBusPendingCallNotifyFunctionFct(void* fct)
 static uintptr_t my_DBusDispatchStatusFunction_fct_##A = 0;   \
 static void my_DBusDispatchStatusFunction_##A(void* connection, int new_status, void* data)     \
 {                                       \
-    RunFunction(my_context, my_DBusDispatchStatusFunction_fct_##A, 3, connection, new_status, data);\
+    RunFunctionFmt(my_DBusDispatchStatusFunction_fct_##A, "pip", connection, new_status, data);\
 }
 SUPER()
 #undef GO
@@ -217,7 +217,7 @@ static void* findDBusDispatchStatusFunctionFct(void* fct)
 static uintptr_t my_DBusAddWatchFunction_fct_##A = 0;   \
 static int my_DBusAddWatchFunction_##A(void* watch, void* data)     \
 {                                       \
-    return (int)RunFunction(my_context, my_DBusAddWatchFunction_fct_##A, 2, watch, data);\
+    return (int)RunFunctionFmt(my_DBusAddWatchFunction_fct_##A, "pp", watch, data);\
 }
 SUPER()
 #undef GO
@@ -240,7 +240,7 @@ static void* findDBusAddWatchFunctionFct(void* fct)
 static uintptr_t my_DBusRemoveWatchFunction_fct_##A = 0;   \
 static void my_DBusRemoveWatchFunction_##A(void* watch, void* data)     \
 {                                       \
-    RunFunction(my_context, my_DBusRemoveWatchFunction_fct_##A, 2, watch, data);\
+    RunFunctionFmt(my_DBusRemoveWatchFunction_fct_##A, "pp", watch, data);\
 }
 SUPER()
 #undef GO
@@ -263,7 +263,7 @@ static void* findDBusRemoveWatchFunctionFct(void* fct)
 static uintptr_t my_DBusWatchToggledFunction_fct_##A = 0;   \
 static void my_DBusWatchToggledFunction_##A(void* watch, void* data)     \
 {                                       \
-    RunFunction(my_context, my_DBusWatchToggledFunction_fct_##A, 2, watch, data);\
+    RunFunctionFmt(my_DBusWatchToggledFunction_fct_##A, "pp", watch, data);\
 }
 SUPER()
 #undef GO
@@ -286,7 +286,7 @@ static void* findDBusWatchToggledFunctionFct(void* fct)
 static uintptr_t my_DBusObjectPathUnregisterFunction_fct_##A = 0;   \
 static void my_DBusObjectPathUnregisterFunction_##A(void* connection, void* data)     \
 {                                       \
-    RunFunction(my_context, my_DBusObjectPathUnregisterFunction_fct_##A, 2, connection, data);\
+    RunFunctionFmt(my_DBusObjectPathUnregisterFunction_fct_##A, "pp", connection, data);\
 }
 SUPER()
 #undef GO
@@ -309,7 +309,7 @@ static void* findDBusObjectPathUnregisterFunctionFct(void* fct)
 static uintptr_t my_DBusObjectPathMessageFunction_fct_##A = 0;   \
 static void my_DBusObjectPathMessageFunction_##A(void* connection, void* message, void* data)     \
 {                                       \
-    RunFunction(my_context, my_DBusObjectPathMessageFunction_fct_##A, 3, connection, message, data);\
+    RunFunctionFmt(my_DBusObjectPathMessageFunction_fct_##A, "ppp", connection, message, data);\
 }
 SUPER()
 #undef GO
@@ -332,7 +332,7 @@ static void* findDBusObjectPathMessageFunctionFct(void* fct)
 static uintptr_t my_dbus_internal_pad_fct_##A = 0;   \
 static void my_dbus_internal_pad_##A(void* a, void* b, void* c, void* d)     \
 {                                       \
-    RunFunction(my_context, my_dbus_internal_pad_fct_##A, 4, a, b, c, d);\
+    RunFunctionFmt(my_dbus_internal_pad_fct_##A, "pppp", a, b, c, d);\
 }
 SUPER()
 #undef GO
@@ -349,6 +349,31 @@ static void* finddbus_internal_padFct(void* fct)
     printf_log(LOG_NONE, "Warning, no more slot for dbus dbus_internal_pad callback\n");
     return NULL;
 }
+
+// DBusNewConnectionFunction
+#define GO(A)   \
+static uintptr_t my_DBusNewConnectionFunction_fct_##A = 0;                      \
+static void my_DBusNewConnectionFunction_##A(void* a, void* b, void* c)         \
+{                                                                               \
+    RunFunctionFmt(my_DBusNewConnectionFunction_fct_##A, "pppp", a, b, c);  \
+}
+SUPER()
+#undef GO
+static void* findDBusNewConnectionFunctionFct(void* fct)
+{
+    if(!fct) return NULL;
+    if(GetNativeFnc((uintptr_t)fct))  return GetNativeFnc((uintptr_t)fct);
+    #define GO(A) if(my_DBusNewConnectionFunction_fct_##A == (uintptr_t)fct) return my_DBusNewConnectionFunction_##A;
+    SUPER()
+    #undef GO
+    #define GO(A) if(my_DBusNewConnectionFunction_fct_##A == 0) {my_DBusNewConnectionFunction_fct_##A = (uintptr_t)fct; return my_DBusNewConnectionFunction_##A; }
+    SUPER()
+    #undef GO
+    printf_log(LOG_NONE, "Warning, no more slot for dbus DBusNewConnectionFunction callback\n");
+    return NULL;
+}
+
+
 #undef SUPER
 
 EXPORT void my_dbus_timeout_set_data(x64emu_t* emu, void* e, void* p, void* f)
@@ -404,7 +429,64 @@ EXPORT int my_dbus_message_append_args_valist(x64emu_t* emu, void* message, int
     #ifdef CONVERT_VALIST
     CONVERT_VALIST(b);
     #else
+    #if 1
     CREATE_VALIST_FROM_VALIST(b, emu->scratch);
+    #else
+    va_list sysv_varargs;
+    uintptr_t *p = (uintptr_t*)(emu->scratch);
+    int n = (X64_VA_MAX_REG - b->gp_offset)/8;
+    int x = (X64_VA_MAX_XMM - b->fp_offset)/8;
+    int idx = 0;
+    int type = arg;
+    while(arg) {
+        if(arg == (int)'d') {
+            // double
+            if(x)
+                *(p++) = *(uintptr_t*)(b->reg_save_area + X64_VA_MAX_XMM - (x--)*8);
+            else
+                *(p++) = *(uintptr_t*)(b->overflow_arg_area + (idx++)*8);
+        } else if(arg == (int)'a') {
+            // array
+            //  type
+            if(n)
+                *(p++) = *(uintptr_t*)(b->reg_save_area + X64_VA_MAX_REG - (n--)*8);
+            else
+                *(p++) = *(uintptr_t*)(b->overflow_arg_area + (idx++)*8);
+            //  elements
+            if(n)
+                *(p++) = *(uintptr_t*)(b->reg_save_area + X64_VA_MAX_REG - (n--)*8);
+            else
+                *(p++) = *(uintptr_t*)(b->overflow_arg_area + (idx++)*8);
+            //  number of elements
+            if(n)
+                *(p++) = *(uintptr_t*)(b->reg_save_area + X64_VA_MAX_REG - (n--)*8);
+            else
+                *(p++) = *(uintptr_t*)(b->overflow_arg_area + (idx++)*8);
+        } else if(arg == (int)'s' || arg == (int)'g' || arg == (int)'o') {
+            //  elements
+            if(n)
+                *(p++) = *(uintptr_t*)(b->reg_save_area + X64_VA_MAX_REG - (n--)*8);
+            else
+                *(p++) = *(uintptr_t*)(b->overflow_arg_area + (idx++)*8);
+            //  number of elements
+            if(n)
+                *(p++) = *(uintptr_t*)(b->reg_save_area + X64_VA_MAX_REG - (n--)*8);
+            else
+                *(p++) = *(uintptr_t*)(b->overflow_arg_area + (idx++)*8);
+        } else {
+            if(n)
+                *(p++) = *(uintptr_t*)(b->reg_save_area + X64_VA_MAX_REG - (n--)*8);
+            else
+                *(p++) = *(uintptr_t*)(b->overflow_arg_area + (idx++)*8);
+        }
+        if(n)
+            arg = *(uintptr_t*)(b->reg_save_area + X64_VA_MAX_REG - (n--)*8);
+        else
+            arg = *(uintptr_t*)(b->overflow_arg_area + (idx++)*8);
+        *(p++) = arg;
+    }
+    sysv_varargs = (va_list)p;
+    #endif
     #endif
     return my->dbus_message_append_args_valist(message, arg, VARARGS);
 }
@@ -478,6 +560,22 @@ EXPORT int my_dbus_connection_try_register_object_path(x64emu_t* emu, void* conn
     return my->dbus_connection_try_register_object_path(connection, path, vtable?&vt:NULL, data, error);
 }
 
+EXPORT int my_dbus_connection_register_fallback(x64emu_t* emu, void* connection, void* path, my_DBusObjectPathVTable_t* vtable, void* data)
+{
+    (void)emu;
+    my_DBusObjectPathVTable_t vt = {0};
+    if(vtable) {
+        vt.unregister_function = findDBusObjectPathUnregisterFunctionFct(vtable->unregister_function);
+        vt.message_function = findDBusObjectPathMessageFunctionFct(vtable->message_function);
+        vt.pad1 = finddbus_internal_padFct(vtable->pad1);
+        vt.pad2 = finddbus_internal_padFct(vtable->pad2);
+        vt.pad3 = finddbus_internal_padFct(vtable->pad3);
+        vt.pad4 = finddbus_internal_padFct(vtable->pad4);
+    }
+
+    return my->dbus_connection_register_fallback(connection, path, vtable?&vt:NULL, data);
+}
+
 EXPORT int my_dbus_connection_set_data(x64emu_t* emu, void* connection, int slot, void* data, void* free_func)
 {
     (void)emu;
@@ -506,7 +604,25 @@ EXPORT int my_dbus_connection_try_register_fallback(x64emu_t* emu, void* connect
     return my->dbus_connection_try_register_fallback(connection, path, vtable?&vt:NULL, data, error);
 }
 
+EXPORT int my_dbus_server_set_watch_functions(x64emu_t* emu, void* server, void* add, void* rem, void* toggle, void* data, void* d)
+{
+    return my->dbus_server_set_watch_functions(server, findDBusAddWatchFunctionFct(add), findDBusRemoveWatchFunctionFct(rem), findDBusWatchToggledFunctionFct(toggle), data, find_DBusFreeFunction_Fct(d));
+}
 
+EXPORT void my_dbus_server_set_new_connection_function(x64emu_t* emu, void* server, void* f, void* data, void* d)
+{
+    my->dbus_server_set_new_connection_function(server, findDBusNewConnectionFunctionFct(f), data, find_DBusFreeFunction_Fct(d));
+}
+
+EXPORT int my_dbus_server_set_timeout_functions(x64emu_t* emu, void* server, void* add, void* rem, void* toggle, void* data, void* d)
+{
+    return my->dbus_server_set_timeout_functions(server, find_DBusAddTimeoutFunction_Fct(add), find_DBusRemoveTimeoutFunction_Fct(rem), find_DBusTimeoutToggledFunction_Fct(toggle), data, find_DBusFreeFunction_Fct(d));
+}
+
+EXPORT int my_dbus_server_set_data(x64emu_t* emu, void* server, int slot, void* data, void* d)
+{
+    return my->dbus_server_set_data(server, slot, data, find_DBusFreeFunction_Fct(d));
+}
 
 #define CUSTOM_INIT \
     getMy(lib);
