@@ -3,15 +3,15 @@
 #endif
 
 //GOM(gdk_add_client_message_filter, vFpBp)
-//GO(gdk_add_option_entries_libgtk_only, 
+GO(gdk_add_option_entries_libgtk_only, vFp)
 //GO(gdk_app_launch_context_get_type, 
-//GO(gdk_app_launch_context_new, 
-//GO(gdk_app_launch_context_set_desktop, 
-//GO(gdk_app_launch_context_set_display, 
-//GO(gdk_app_launch_context_set_icon, 
-//GO(gdk_app_launch_context_set_icon_name, 
-//GO(gdk_app_launch_context_set_screen, 
-//GO(gdk_app_launch_context_set_timestamp, 
+GO(gdk_app_launch_context_new, pFv)
+GO(gdk_app_launch_context_set_desktop, vFpi)
+GO(gdk_app_launch_context_set_display, vFpp)
+GO(gdk_app_launch_context_set_icon, vFpp)
+GO(gdk_app_launch_context_set_icon_name, vFpp)
+GO(gdk_app_launch_context_set_screen, vFpp)
+GO(gdk_app_launch_context_set_timestamp, vFpu)
 GO(gdk_atom_intern, pFpi)
 GO(gdk_atom_intern_static_string, pFp)
 GO(gdk_atom_name, pFp)
@@ -79,25 +79,28 @@ GO(gdk_cursor_ref, pFp)
 GO(gdk_cursor_type_get_type, iFv)
 GO(gdk_cursor_unref, vFp)
 GO(gdk_device_free_history, vFpi)
-GO(gdk_device_get_axis, iFppip)
-GO(gdk_device_get_axis_use, iFpu)
+GO(gdk_device_get_axis, iFppup)
+GO(gdk_device_get_axis_use, uFpu)
 GO(gdk_device_get_core_pointer, pFv)
 GO(gdk_device_get_has_cursor, iFp)
 GO(gdk_device_get_history, iFppuupp)
-GO(gdk_device_get_key, vFpupp)
-GO(gdk_device_get_mode, iFp)
+GO(gdk_device_get_key, iFpupp)
+GO(gdk_device_get_mode, uFp)
 GO(gdk_device_get_name, pFp)
 GO(gdk_device_get_n_axes, iFp)
 GO(gdk_device_get_n_keys, iFp)
-GO(gdk_device_get_source, iFp)
+GO(gdk_device_get_position, vFpppp)
+GO(gdk_device_get_source, uFp)
 GO(gdk_device_get_state, vFpppp)
 GO(gdk_device_get_type, iFv)
-GO(gdk_device_set_axis_use, vFpui)
+GO(gdk_device_set_axis_use, vFpuu)
+GO(gdk_device_manager_get_client_pointer, pFp)
 GO(gdk_device_manager_get_type, lFv)
-GO(gdk_device_set_key, vFpuui)
-GO(gdk_device_set_mode, iFpi)
+GO(gdk_device_set_key, vFpuuu)
+GO(gdk_device_set_mode, iFpu)
 GO(gdk_device_set_source, vFpi)
 GO(gdk_devices_list, pFv)
+GO(gdk_device_ungrab, vFpu)
 //GOM(gdk_display_add_client_message_filter, vFEpppp)
 GO(gdk_display_beep, vFp)
 GO(gdk_display_close, vFp)
@@ -108,11 +111,16 @@ GO(gdk_display_get_default_cursor_size, uFp)
 GO(gdk_display_get_default_group, pFp)
 GO(gdk_display_get_default_screen, pFp)
 GO(gdk_display_get_default_seat, pFp)
+GO(gdk_display_get_device_manager, pFp)
 GO(gdk_display_get_event, pFp)
 GO(gdk_display_get_maximal_cursor_size, vFppp)
 GO(gdk_display_get_name, pFp)
+GO(gdk_display_get_monitor, pFpi)
+GO(gdk_display_get_monitors, pFp)
+GO(gdk_display_get_n_monitors, iFp)
 GO(gdk_display_get_n_screens, iFp)
 GO(gdk_display_get_pointer, vFppppp)
+GO(gdk_display_get_primary_monitor, pFp)
 GO(gdk_display_get_screen, pFpi)
 GO(gdk_display_get_type, iFv)
 GO(gdk_display_get_window_at_pointer, pFppp)
@@ -127,7 +135,7 @@ GO(gdk_display_manager_set_default_display, vFpp)
 GO(gdk_display_map_keycode, iFpuppp)
 GO(gdk_display_notify_startup_complete, vFpp)
 GO(gdk_display_open, pFp)
-//GO(gdk_display_open_default_libgtk_only, 
+GO(gdk_display_open_default_libgtk_only, pFv)
 GO(gdk_display_peek_event, pFp)
 GO(gdk_display_pointer_is_grabbed, iFp)
 GO(gdk_display_pointer_ungrab, vFpu)
@@ -151,8 +159,8 @@ GO(gdk_drag_action_get_type, iFv)
 GO(gdk_drag_begin, pFpp)
 GO(gdk_drag_context_get_actions, iFp)
 GO(gdk_drag_context_get_dest_window, pFp)
-GO(gdk_drag_context_get_protocol, iFp)
-GO(gdk_drag_context_get_selected_action, iFp)
+GO(gdk_drag_context_get_protocol, uFp)
+GO(gdk_drag_context_get_selected_action, uFp)
 GO(gdk_drag_context_get_source_window, pFp)
 GO(gdk_drag_context_get_suggested_action, iFp)
 GO(gdk_drag_context_get_type, iFv)
@@ -167,9 +175,9 @@ GO(gdk_drag_find_window_for_screen, vFpppiipp)
 GO(gdk_drag_get_protocol, pFpp)
 GO(gdk_drag_get_protocol_for_display, pFppp)
 GO(gdk_drag_get_selection, pFp)
-GO(gdk_drag_motion, iFppiiiiiu)
+GO(gdk_drag_motion, iFppuiiuuu)
 GO(gdk_drag_protocol_get_type, iFv)
-GO(gdk_drag_status, vFpiu)
+GO(gdk_drag_status, vFpuu)
 GO(gdk_drawable_copy_to_image, pFppiiiiii)
 GO(gdk_drawable_get_clip_region, pFp)
 GO(gdk_drawable_get_colormap, pFp)
@@ -220,13 +228,14 @@ GO(gdk_error_trap_push, vFv)
 GO(gdk_event_copy, pFp)
 GO(gdk_event_free, vFp)
 GO(gdk_event_get, pFv)
-GO(gdk_event_get_axis, iFpip)
+GO(gdk_event_get_axis, iFpup)
 GO(gdk_event_get_coords, iFppp)
 GO(gdk_event_get_graphics_expose, pFp)
 GO(gdk_event_get_modifier_state, iFp)
 GO(gdk_event_get_root_coords, iFppp)
 GO(gdk_event_get_screen, pFp)
 GO(gdk_event_get_state, iFpp)
+GO(gdk_event_get_source_device, pFp)
 GO(gdk_event_get_surface, pFp)
 GO(gdk_event_get_time, uFp)
 GO(gdk_event_get_type, iFv)
@@ -304,6 +313,11 @@ GO(gdk_get_display_arg_name, pFv)
 GO(gdk_get_program_class, pFv)
 GO(gdk_get_show_events, iFv)
 GO(gdk_get_use_xshm, iFv)
+GO(gdk_gl_context_get_type, LFv)
+GO(gdk_gl_context_clear_current, vFv)
+GO(gdk_gl_context_get_current, pFv)
+GO(gdk_gl_context_make_current, vFp)
+GO(gdk_gl_context_realize, iFpp)
 GO(gdk_grab_status_get_type, iFv)
 GO(gdk_gravity_get_type, iFv)
 GO(gdk_image_get, pFpiiii)
@@ -339,13 +353,13 @@ GO(gdk_input_source_get_type, iFv)
 GO(gdk_join_style_get_type, iFv)
 GO(gdk_key_event_get_keycode, uFp)
 GO(gdk_key_event_get_keyval, uFp)
-GO(gdk_keyboard_grab, iFpiu)
+GO(gdk_keyboard_grab, uFpiu)
 //GO(gdk_keyboard_grab_info_libgtk_only, 
 GO(gdk_keyboard_ungrab, vFu)
 GO(gdk_keymap_add_virtual_modifiers, vFpp)
 GO(gdk_keymap_get_caps_lock_state, iFp)
 GO(gdk_keymap_get_default, pFv)
-GO(gdk_keymap_get_direction, iFp)
+GO(gdk_keymap_get_direction, uFp)
 GO(gdk_keymap_get_entries_for_keycode, iFpuppp)
 GO(gdk_keymap_get_entries_for_keyval, iFpupp)
 GO(gdk_keymap_get_for_display, pFp)
@@ -353,7 +367,7 @@ GO(gdk_keymap_get_type, iFv)
 GO(gdk_keymap_have_bidi_layouts, iFp)
 GO(gdk_keymap_lookup_key, uFpp)
 GO(gdk_keymap_map_virtual_modifiers, iFpp)
-GO(gdk_keymap_translate_keyboard_state, iFpuiipppp)
+GO(gdk_keymap_translate_keyboard_state, iFpuuipppp)
 GO(gdk_keyval_convert_case, vFupp)
 GO(gdk_keyval_from_name, uFp)
 GO(gdk_keyval_is_lower, iFu)
@@ -365,6 +379,18 @@ GO(gdk_keyval_to_upper, uFu)
 GO(gdk_line_style_get_type, iFv)
 GO(gdk_list_visuals, pFv)
 //GO(gdk_mbstowcs, 
+GO(gdk_monitor_get_connector, pFp)
+GO(gdk_monitor_get_description, pFp)
+GO(gdk_monitor_get_display, pFp)
+GO(gdk_monitor_get_geometry, vFpp)
+GO(gdk_monitor_get_height_mm, iFp)
+GO(gdk_monitor_get_manufacturer, pFp)
+GO(gdk_monitor_get_model, pFp)
+GO(gdk_monitor_get_refresh_rate, iFp)
+GO(gdk_monitor_get_scale_factor, iFp)
+GO(gdk_monitor_get_subpixel_layout, uFp)
+GO(gdk_monitor_get_width_mm, iFp)
+GO(gdk_monitor_is_valid, iFp)
 GO(gdk_memory_texture_new, pFiiipL)
 GO(gdk_modifier_type_get_type, iFv)
 GO(gdk_net_wm_supports, iFp)
@@ -418,12 +444,12 @@ GO(gdk_pixmap_impl_x11_get_type, iFv)
 GO(gdk_pixmap_lookup, pFp)
 GO(gdk_pixmap_lookup_for_display, pFpp)
 GO(gdk_pixmap_new, pFpiii)
-GO(gdk_pointer_grab, iFpiippu)
+GO(gdk_pointer_grab, uFpiuppu)
 //GO(gdk_pointer_grab_info_libgtk_only, 
 GO(gdk_pointer_is_grabbed, iFv)
 GO(gdk_pointer_ungrab, vFu)
 //GO(gdk_pre_parse_libgtk_only, 
-GO(gdk_property_change, vFpppiipi)
+GO(gdk_property_change, vFpppiupi)
 GO(gdk_property_delete, vFpp)
 GO(gdk_property_get, iFpppLLipppp)
 GO(gdk_property_state_get_type, iFv)
@@ -453,7 +479,10 @@ GO(gdk_region_subtract, vFpp)
 GO(gdk_region_union, vFpp)
 GO(gdk_region_union_with_rect, vFpp)
 GO(gdk_region_xor, vFpp)
-GO(gdk_rgba_get_type, lFv)
+GO(gdk_rgba_free, vFp)
+GO(gdk_rgba_get_type, LFv)
+GO(gdk_rgba_parse, iFpp)
+GO(gdk_rgba_to_string, pFp)
 GO(gdk_rgb_cmap_free, vFp)
 GO(gdk_rgb_cmap_new, pFpi)
 GO(gdk_rgb_colormap_ditherable, iFp)
@@ -483,6 +512,7 @@ GO(gdk_screen_get_monitor_geometry, vFpip)
 GO(gdk_screen_get_monitor_height_mm, iFpi)
 GO(gdk_screen_get_monitor_plug_name, pFpi)
 GO(gdk_screen_get_monitor_width_mm, iFpi)
+GO(gdk_screen_get_monitor_workarea, vFpip)
 GO(gdk_screen_get_n_monitors, iFp)
 GO(gdk_screen_get_number, iFp)
 GO(gdk_screen_get_primary_monitor, iFp)
@@ -543,7 +573,7 @@ GO(gdk_string_to_compound_text_for_display, iFpppppp)
 //GO(gdk_subwindow_mode_get_type, 
 //GO(gdk_synthesize_window_state, 
 //GO(gdk_test_render_sync, 
-//GO(gdk_test_simulate_button, 
+GO(gdk_test_simulate_button, iFpiiuui)
 //GO(gdk_test_simulate_key, 
 //GO(gdk_text_extents, 
 //GO(gdk_text_extents_wc, 
@@ -552,16 +582,16 @@ GO(gdk_string_to_compound_text_for_display, iFpppppp)
 GO(gdk_text_property_to_text_list, iFpipip)
 GO(gdk_text_property_to_text_list_for_display, iFppipip)
 GO(gdk_text_property_to_utf8_list, iFpipip)
-//GO(gdk_text_property_to_utf8_list_for_display, 
+GO(gdk_text_property_to_utf8_list_for_display, iFppipip)
 //GO(gdk_text_width, 
 //GO(gdk_text_width_wc, 
 GO(gdk_texture_download, vFppL)
 GO(gdk_texture_get_width, uFp)
 GO(gdk_texture_get_height, uFp)
-//GOM(gdk_threads_add_idle, uFEpp)
-//GOM(gdk_threads_add_idle_full, uFEippp)
+GOM(gdk_threads_add_idle, uFEpp)
+GOM(gdk_threads_add_idle_full, uFEippp)
 //GOM(gdk_threads_add_timeout, uFEupp)
-//GOM(gdk_threads_add_timeout_full, uFEiuppp)
+GOM(gdk_threads_add_timeout_full, uFEiuppp)
 //GOM(gdk_threads_add_timeout_seconds, uFEupp)
 //GOM(gdk_threads_add_timeout_seconds_full, uFEiuppp)
 GO(gdk_threads_enter, vFv)
@@ -576,23 +606,24 @@ GO(gdk_utf8_to_string_target, pFp)
 GO(gdk_visibility_state_get_type, iFv)
 GO(gdk_visual_get_best, pFv)
 GO(gdk_visual_get_best_depth, iFv)
-GO(gdk_visual_get_best_type, iFv)
-GO(gdk_visual_get_best_with_both, pFii)
+GO(gdk_visual_get_best_type, uFv)
+GO(gdk_visual_get_best_with_both, pFiu)
 GO(gdk_visual_get_best_with_depth, pFi)
-GO(gdk_visual_get_best_with_type, pFi)
+GO(gdk_visual_get_best_with_type, pFu)
 GO(gdk_visual_get_bits_per_rgb, iFp)
 GO(gdk_visual_get_blue_pixel_details, vFpppp)
-GO(gdk_visual_get_byte_order, iFp)
+GO(gdk_visual_get_byte_order, uFp)
 GO(gdk_visual_get_colormap_size, iFp)
 GO(gdk_visual_get_depth, iFp)
 GO(gdk_visual_get_green_pixel_details, vFpppp)
 GO(gdk_visual_get_red_pixel_details, vFpppp)
 GO(gdk_visual_get_screen, pFp)
 GO(gdk_visual_get_system, pFv)
-GO(gdk_visual_get_type, iFv)
-GO(gdk_visual_get_visual_type, iFp)
+GO(gdk_visual_get_type, LFv)
+GO(gdk_visual_get_visual_type, uFp)
 GO(gdk_visual_type_get_type, iFv)
 GO(gdk_wayland_toplevel_set_transient_for_exported, iFpp)
+GO(gdk_wayland_window_get_type, LFv)
 GO(gdk_wayland_window_set_transient_for_exported, iFpp)
 //GO(gdk_wcstombs, 
 GOM(gdk_window_add_filter, vFEppp)
@@ -611,8 +642,9 @@ GO(gdk_window_configure_finished, vFp)
 GO(gdk_window_constrain_size, vFpuiipp)
 GO(gdk_window_coords_from_parent, vFpddpp)
 GO(gdk_window_coords_to_parent, vFpddpp)
+GO(gdk_window_create_gl_context, pFpp)
 GO(gdk_window_create_similar_image_surface, pFpiiii)
-GO(gdk_window_create_similar_surface, pFpiii)
+GO(gdk_window_create_similar_surface, pFpuii)
 GO(gdk_window_deiconify, vFp)
 GO(gdk_window_destroy, vFp)
 //GO(gdk_window_destroy_notify, 
@@ -638,10 +670,10 @@ GO(gdk_window_get_deskrelative_origin, iFppp)
 GO(gdk_window_get_display, pFp)
 GO(gdk_window_get_effective_parent, pFp)
 GO(gdk_window_get_effective_toplevel, pFp)
-GO(gdk_window_get_events, vFpi)
+GO(gdk_window_get_events, uFp)
 GO(gdk_window_get_focus_on_map, iFp)
 GO(gdk_window_get_frame_extents, vFpp)
-GO(gdk_window_get_geometry, vFpppppp)
+GO(gdk_window_get_geometry, vFppppp)
 GO(gdk_window_get_group, pFp)
 GO(gdk_window_get_height, iFp)
 GO(gdk_window_get_internal_paint_info, vFpppp)
@@ -653,16 +685,16 @@ GO(gdk_window_get_position, vFppp)
 GO(gdk_window_get_root_coords, vFpiipp)
 GO(gdk_window_get_root_origin, vFppp)
 GO(gdk_window_get_screen, pFp)
-GO(gdk_window_get_state, iFp)
+GO(gdk_window_get_state, uFp)
 GO(gdk_window_get_toplevel, pFp)
 GO(gdk_window_get_toplevels, pFp)
 GO(gdk_window_get_type, lFv)
-GO(gdk_window_get_type_hint, iFp)
+GO(gdk_window_get_type_hint, uFp)
 GO(gdk_window_get_update_area, pFp)
 GO(gdk_window_get_user_data, vFpp)
 GO(gdk_window_get_visual, pFp)
 GO(gdk_window_get_width, iFp)
-GO(gdk_window_get_window_type, iFp)
+GO(gdk_window_get_window_type, uFp)
 GO(gdk_window_has_native, iFp)
 GO(gdk_window_hide, vFp)
 GO(gdk_window_hints_get_type, iFv)
@@ -704,17 +736,18 @@ GO(gdk_window_restack, vFppi)
 GO(gdk_window_scroll, vFpii)
 GO(gdk_window_set_accept_focus, vFpi)
 GO(gdk_window_set_background, vFpp)
+GO(gdk_window_set_background_rgba, vFpp)
 GO(gdk_window_set_back_pixmap, vFppi)
 GO(gdk_window_set_child_input_shapes, vFp)
 GO(gdk_window_set_child_shapes, vFp)
 GO(gdk_window_set_composited, vFpi)
 GO(gdk_window_set_cursor, vFpp)
 GO(gdk_window_set_debug_updates, vFi)
-GO(gdk_window_set_decorations, vFpp)
-GO(gdk_window_set_events, vFpp)
+GO(gdk_window_set_decorations, vFpu)
+GO(gdk_window_set_events, vFpu)
 GO(gdk_window_set_focus_on_map, vFpi)
 GO(gdk_window_set_functions, vFpu)
-GO(gdk_window_set_geometry_hints, vFppi)
+GO(gdk_window_set_geometry_hints, vFppu)
 GO(gdk_window_set_group, vFpp)
 GO(gdk_window_set_hints, vFpiiiiiii)
 GO(gdk_window_set_icon, vFpppp)
@@ -723,7 +756,7 @@ GO(gdk_window_set_icon_name, vFpp)
 GO(gdk_window_set_keep_above, vFpi)
 GO(gdk_window_set_keep_below, vFpi)
 GO(gdk_window_set_modal_hint, iFp)
-GO(gdk_window_set_opacity, vFpi)
+GO(gdk_window_set_opacity, vFpd)
 GO(gdk_window_set_override_redirect, vFpi)
 GO(gdk_window_set_role, vFpp)
 GO(gdk_window_set_skip_pager_hint, vFpi)
@@ -732,7 +765,7 @@ GO(gdk_window_set_startup_id, vFpp)
 GO(gdk_window_set_static_gravities, iFpi)
 GO(gdk_window_set_title, vFpp)
 GO(gdk_window_set_transient_for, vFpp)
-GO(gdk_window_set_type_hint, vFpi)
+GO(gdk_window_set_type_hint, vFpu)
 GO(gdk_window_set_urgency_hint, vFpi)
 GO(gdk_window_set_user_data, vFpp)
 GO(gdk_window_shape_combine_mask, vFppii)
@@ -763,6 +796,7 @@ GO(gdk_x11_display_get_startup_notification_id, pFp)
 GO(gdk_x11_display_get_type, iFv)
 GO(gdk_x11_display_get_user_time, uFp)
 GO(gdk_x11_display_get_xdisplay, pFp)
+GO(gdk_x11_window_get_type, LFv)
 GO(gdk_x11_window_get_xid, LFp)
 GO(gdk_x11_display_grab, vFp)
 GO(gdk_x11_display_set_cursor_theme, vFppi)
@@ -820,4 +854,4 @@ DATA(gdk_threads_lock, 4)
 DATA(gdk_threads_unlock, 4)
 
 GO(dummy_set_child_property, vFppupp)       // to get vFppupp for gtkclass.c
-GO(dummy_constructor, vFLup)    // to get vFlup for gtkclass.c
\ No newline at end of file
+GO(dummy_constructor, vFLup)    // to get vFlup for gtkclass.c
