@@ -14,6 +14,7 @@
 #include "emu/x64emu_private.h"
 #include "elfloader.h"
 #include "box64context.h"
+#include "x64tls.h"
 
 typedef struct my_tls_s {
     unsigned long int   i;
@@ -23,8 +24,14 @@ typedef struct my_tls_s {
 EXPORT void* my___tls_get_addr(void* p)
 {
     my_tls_t *t = (my_tls_t*)p;
-    void* ret = (void*)((char*)GetTLSPointer(my_context, my_context->elfs[t->i])+t->o);
-    return ret;
+    tlsdatasize_t* ptr = getTLSData(my_context);
+    return ptr->data+GetTLSBase(my_context->elfs[t->i])+t->o;
+}
+
+EXPORT void* my___libc_stack_end;
+void stSetup(box64context_t* context)
+{
+    my___libc_stack_end = context->stack;   // is this the end, or should I add stasz?
 }
 
 // don't try to load the actual ld-linux (because name is variable), just use box64 itself, as it's linked to ld-linux
@@ -36,6 +43,9 @@ const char* ldlinuxName = "ld-linux.so.2";
         lib->w.lib = dlopen(NULL, RTLD_LAZY | RTLD_GLOBAL);    \
     else
 
+#define CUSTOM_INIT         \
+    stSetup(box64);         \
+
 // define all standard library functions
 #include "wrappedlib_init.h"
 
