@@ -24,7 +24,7 @@ GO(__adjtimex, iFp)
 GOW(adjtimex, iFp)
 //GOW(advance, 
 GO(alarm, uFu)
-GOW(aligned_alloc, pFLL)
+GO(aligned_alloc, pFUU)
 GOW(alphasort, iFpp)
 GOW(alphasort64, iFpp)
 //GO(__arch_prctl, 
@@ -94,9 +94,9 @@ GOW(brk, iFp)
 GOM(bsearch, pFEppLLp)
 GOW(btowc, uFi)
 GO(__bzero, vFpL)
-GOW(bzero, vFpL)
+GO(bzero, vFpL)
 //GO(c16rtomb, LFpW!)
-//GOW(c32rtomb, LFpu!)
+GOW(c32rtomb, LFpup)
 GOW(calloc, pFLL)
 //GO(callrpc, iFpLLL@p@p)
 //GO(__call_tls_dtors, vFv)
@@ -177,7 +177,7 @@ GO(ctime_r, pFpp)
 //DATA(__ctype32_toupper, 
 DATA(__ctype_b, sizeof(void*))
 GO(__ctype_b_loc, pFv)
-GOW(__ctype_get_mb_cur_max, LFv)
+GO(__ctype_get_mb_cur_max, LFv)
 //GO(__ctype_init, 
 DATAM(__ctype_tolower, sizeof(void*))
 GO(__ctype_tolower_loc, pFv)
@@ -219,8 +219,8 @@ GOWM(dl_iterate_phdr, iFEpp)
 //GO(_dl_sym, 
 //GO(_dl_vsym, 
 GOW(dngettext, pFpppL)
-//GO(dprintf, iFipV)
-//GO(__dprintf_chk, 
+GOM(dprintf, iFEipV)
+GOM(__dprintf_chk, iFEiipV)
 GO(drand48, dFv)
 GO(drand48_r, iFpp)
 GOW(dup, iFi)
@@ -294,7 +294,7 @@ GO(eventfd, iFui)
 GO(eventfd_read, iFip)
 GO(eventfd_write, iFiL)
 GOM(execl, iFEpV) // First argument is also part of the variadic
-//GOM(execle, iFEpV) // First argument is also part of the variadic
+GOM(execle, iFEpV) // First argument is also part of the variadic
 GOM(execlp, iFEpV) // First argument is also part of the variadic
 GOM(execv, iFEpp)
 GOWM(execve, iFEppp)
@@ -324,7 +324,7 @@ GOWM(__fcntl, iFEiiN)
 GOWM(fcntl, iFEiiN)
 GOWM(fcntl64, iFEiiN)
 GO(fcvt, pFdipp)
-GOW(fcvt_r, iFdipppL)
+GO(fcvt_r, iFdipppL)
 GO(fdatasync, iFi)
 GO(__fdelt_chk, LFL)
 //GO(__fdelt_warn, 
@@ -341,7 +341,7 @@ GOW(fflush, iFS)
 GO(fflush_unlocked, iFS)
 //GO(__ffs, 
 GO(ffs, iFi)
-GOW(ffsl, iFl)
+GO(ffsl, iFI)
 GO(ffsll, iFI)
 GOW(fgetc, iFp)
 GOW(fgetc_unlocked, iFp)
@@ -366,8 +366,8 @@ GO(fgetws, pFpip)
 GO(fgetws_unlocked, pFpip)
 //GO(__fgetws_unlocked_chk, 
 GO(fgetxattr, lFippL)
-GOW(fileno, iFp)
-GOW(fileno_unlocked, iFp)
+GOW(fileno, iFS)
+GOW(fileno_unlocked, iFS)
 GO(__finite, iFd)
 GOW(finite, iFd)
 GO(__finitef, iFf)
@@ -384,11 +384,11 @@ GO(fmtmsg, iFlpippp)
 GO(fnmatch, iFppi)
 GOM(fopen, pFEpp)
 GOWM(fopen64, pFEpp)
-//GO(fopencookie, pFpp?)
+GOM(fopencookie, pFEppV)
 //GO(__fork, 
 GOWM(fork, iFEv)
 //GO(__fortify_fail, 
-GOW(fpathconf, lFii)
+GO(fpathconf, lFii)
 GO(__fpending, LFp)
 GOM(fprintf, iFEppV)
 GOM(__fprintf_chk, iFEpipV)
@@ -402,11 +402,11 @@ GO(fputwc, uFip)
 GO(fputwc_unlocked, uFip)
 GO(fputws, iFpp)
 GO(fputws_unlocked, iFpp)
-GOW(fread, LFpLLp)
+GO(fread, LFpLLp)
 GO(__freadable, iFp)
 GO(__fread_chk, LFpLLLp)
 GO(__freading, iFp)
-GOW(fread_unlocked, LFpLLp)
+GO(fread_unlocked, LFpLLp)
 GO(__fread_unlocked_chk, LFpLLLp)
 GO(free, vFp)
 GO(freeaddrinfo, vFp)
@@ -421,9 +421,9 @@ GOW(frexpf, fFfp)
 GOW(frexpl, DFDp)
 GOM(fscanf, iFEppV)
 GO(fseek, iFpli)
-GOW(fseeko, iFpli)
+GO(fseeko, iFpIi)
 GO(__fseeko64, iFpli)
-GOW(fseeko64, iFpli)
+GO(fseeko64, iFpli)
 GO(__fsetlocking, iFpi)
 GO(fsetpos, iFpp)
 GO(fsetpos64, iFpp)
@@ -437,14 +437,14 @@ GOW(fstatfs64, iFip)
 GOW(fstatvfs, iFip)
 GOW(fstatvfs64, iFip)
 GO(fsync, iFi)
-GOW(ftell, lFp)
-GOW(ftello, lFp)
+GO(ftell, lFp)
+GO(ftello, IFp)
 GO(__ftello64, lFp)
-GOW(ftello64, lFp)
+GO(ftello64, lFp)
 GO(ftime, iFp)
 GO(ftok, iFpi)
-GOW(ftruncate, iFil)
-GOW(ftruncate64, iFil)
+GO(ftruncate, iFiI)
+GO(ftruncate64, iFil)
 GOW(ftrylockfile, iFp)
 GOW(fts64_children, pFpi)
 GOW(fts64_close, iFp)
@@ -466,7 +466,7 @@ GO(fwide, iFpi)
 GOWM(fwprintf, iFEppV)
 GOM(__fwprintf_chk, iFEpipV)
 GO(__fwritable, iFp)
-GOW(fwrite, LFpLLp)
+GO(fwrite, LFpLLp)
 GO(fwrite_unlocked, LFpLLp)
 GO(__fwriting, iFp)
 //GO(fwscanf, iFppV)
@@ -496,16 +496,16 @@ GOWM(getcontext, iFEp)
 GOW(getcpu, iFpp)
 GOW(getc_unlocked, iFp)
 GO(get_current_dir_name, pFv)
-GOW(getcwd, pFpL)
+GO(getcwd, pFpL)
 GO(__getcwd_chk, pFpLL)
 GO(getdate, pFp)
 //DATAB(getdate_err, 
 GOW(getdate_r, iFpp)
 GOW(__getdelim, lFppip)
-GOW(getdelim, lFppip)
-GOW(getdents64, lFipL)
-GOW(getdirentries, lFipLp)
-GOW(getdirentries64, lFipLp)
+GO(getdelim, lFppip)
+GO(getdents64, lFipL)
+GO(getdirentries, IFipUp)
+GO(getdirentries64, lFipLp)
 GO(getdomainname, iFpL)
 //GO(__getdomainname_chk, 
 GOW(getdtablesize, iFv)
@@ -541,7 +541,7 @@ GOW(getifaddrs, iFp)
 //GO(getipv4sourcefilter, iFi??pp!)
 GOW(getitimer, iFup)
 //GO(get_kernel_syms, // Deprecated
-GOW(getline, lFppp)
+GO(getline, lFppp)
 GO(getloadavg, iFpi)
 GO(getlogin, pFv)
 GOW(getlogin_r, iFpL)
@@ -675,7 +675,7 @@ GOW(group_member, iFu)
 GOW(gsignal, iFi)
 //GO(gtty, // Deprecated
 GOW(hasmntopt, pFpp)
-GO(hcreate, iFL)
+GO(hcreate, iFU)
 //GOW(hcreate_r, iFL!)
 GOW(hdestroy, vFv)
 //GOW(hdestroy_r, vF!)
@@ -738,8 +738,8 @@ GOW(inet_pton, iFipp)
 //GO(__inet_pton_length, 
 GO(initgroups, iFpu)
 //GO(init_module, // Deprecated
-GOW(initstate, pFupL)
-GOW(initstate_r, iFupLp)
+GO(initstate, pFupL)
+GO(initstate_r, iFupLp)
 GO(innetgr, iFpppp)
 GO(inotify_add_watch, iFipu)
 GO(inotify_init, iFv)
@@ -925,16 +925,20 @@ GO(__isnanf, iFf)
 GOW(isnanf, iFf)
 GO(__isnanl, iFD)
 GOW(isnanl, iFD)
+GO2(__isoc23_fscanf, iFEppV, my___isoc99_fscanf)
+GO2(__isoc23_sscanf, iFEppV, my___isoc99_sscanf)
+GO2(__isoc23_strtol, lFppi, strtol)
+GO2(__isoc23_strtoul, LFppi, strtoul)
 GOM(__isoc99_fscanf, iFEppV)
 //GO(__isoc99_fwscanf, iFppV)
 GOM(__isoc99_scanf, iFEpV)
 GOM(__isoc99_sscanf, iFEppV)
-//GO(__isoc99_swscanf, iFppV)
-GOM(__isoc99_vfscanf, iFEppp)
+GOM(__isoc99_swscanf, iFEppV)
+GOM(__isoc99_vfscanf, iFEppA)
 //GO(__isoc99_vfwscanf, iFppA)
 //GO(__isoc99_vscanf, iFpA)
-GOM(__isoc99_vsscanf, iFEppp)
-//GO(__isoc99_vswscanf, iFppA)
+GOM(__isoc99_vsscanf, iFEppA)
+GOM(__isoc99_vswscanf, iFEppA)
 //GO(__isoc99_vwscanf, iFpA)
 //GO(__isoc99_wscanf, iFpV)
 GO(isprint, iFi)
@@ -1095,8 +1099,8 @@ GO(localtime, pFp)
 GO2(localtime64, pFp, localtime)
 GOW(localtime_r, pFpp)
 GO2(localtime64_r, pFpp, localtime_r)   //Weak
-GO(lockf, iFiil)
-GOW(lockf64, iFiil)
+GO(lockf, iFiiI)
+GO(lockf64, iFiil)
 //DATAB(locs, 
 GOWM(_longjmp, vFEpi)
 GOWM(longjmp, vFEpi)
@@ -1106,8 +1110,8 @@ GO(lrand48_r, iFpp)
 GO(lremovexattr, iFpp)
 GOM(lsearch, pFEpppLp)
 GO(__lseek, IFiIi)
-GOW(lseek, lFili)
-GOW(lseek64, lFili)
+GO(lseek, IFiIi)
+GO(lseek64, lFili)
 GO(lsetxattr, iFpppLi)
 GOM(lstat, iFEpp)
 GOM(lstat64, iFEpp)
@@ -1118,29 +1122,29 @@ GO(__madvise, iFpLi)
 GOW(madvise, iFpLi)
 GOWM(makecontext, iFEppiV)
 GOWM(mallinfo, pFEp)
-GO(malloc, pFL) // need to wrap to clear allocated memory?
+GO(malloc, pFL)
 //GO(malloc_get_state, // Obsolete
 GOW(malloc_info, iFip)
 //GO(malloc_set_state, // Obsolete
 GOW(malloc_stats, vFv)
-GOW(malloc_trim, iFL)
-GOW(malloc_usable_size, LFp)
+GO(malloc_trim, iFL)
+GO(malloc_usable_size, LFp)
 GOW(mallopt, iFii)
 //DATAB(mallwatch, 
-GO(mblen, iFpL)
+GO(mblen, iFpU)
 GO(__mbrlen, LFpLp)
-GOW(mbrlen, LFpLp)
+GO(mbrlen, LFpLp)
 //GO(mbrtoc16, LFppL!)
-//GOW(mbrtoc32, LFppL!)
+GOW(mbrtoc32, LFppLp)
 GO(__mbrtowc, LFppLp)
-GOW(mbrtowc, LFppLp)
+GO(mbrtowc, LFppLp)
 GOW(mbsinit, iFp)
-GOW(mbsnrtowcs, LFppLLp)
-//GO(__mbsnrtowcs_chk, 
-GOW(mbsrtowcs, LFppLp)
-//GO(__mbsrtowcs_chk, 
+GO(mbsnrtowcs, LFppLLp)
+GO(__mbsnrtowcs_chk, LFppLLpL)
+GO(mbsrtowcs, LFppLp)
+GO(__mbsrtowcs_chk, LFppLpL)
 GO(mbstowcs, LFppL)
-//GO(__mbstowcs_chk, 
+GO(__mbstowcs_chk, LFppLL)
 GO(mbtowc, iFppL)
 //GO(mcheck, iF@)
 GO(mcheck_check_all, vFv)
@@ -1148,21 +1152,22 @@ GO(mcheck_check_all, vFv)
 //GO(_mcleanup, 
 GO(_mcount, vFpp)
 GO2(mcount, vFpp, _mcount)
-GOW(memalign, pFLL)
-GOW(memccpy, pFppiL)
+GO(memalign, pFLL)
+GO(memccpy, pFppiL)
 GO(memchr, pFpiL)
 GO(memcmp, iFppL)
+GO(__memcmpeq, iFppL)
 GO(memcpy, pFppL)
 GO(__memcpy_chk, pFppuL)
 GO(memfd_create, iFpu)
 GO(memfrob, pFpL)
-GOW(memmem, pFpLpL)
-GO(memmove, pFppL)
+GO(memmem, pFpLpL)
+GO(memmove, pFppU)
 GO(__memmove_chk, pFppLL)
 GO(__mempcpy, pFppL)
 GO(mempcpy, pFppL)
 GO(__mempcpy_chk, pFppLL)
-GOW(memrchr, pFpiL)
+GO(memrchr, pFpiL)
 GO(memset, pFpiL)
 GO(__memset_chk, pFpiLL)
 //GO(__mempcpy_small, 
@@ -1173,8 +1178,8 @@ GO(mkdirat, iFipu)
 GO(mkdtemp, pFp)
 GO(mkfifo, iFpu)
 GO(mkfifoat, iFipu)
-GO(mknod, iFpuu)
-GO(mknodat, iFipuu)
+GO(mknod, iFpuL)
+GO(mknodat, iFipuL)
 GO(mkostemp, iFpi)
 GOW(mkostemp64, iFpi)
 GO(mkostemps, iFpii)
@@ -1204,7 +1209,7 @@ GOW(mount, iFpppLp)
 GO(mprobe, iFp)
 //GO(__mprotect, 
 GOWM(mprotect, iFEpLi)
-GO(mrand48, lFv)
+GO(mrand48, IFv)
 GO(mrand48_r, iFpp)
 GOM(mremap, pFEpLLiN) //weal
 GO(msgctl, iFiip)
@@ -1291,7 +1296,7 @@ GOW(openat, iFipON)
 GO(__openat_2, iFipO)
 GOW(openat64, iFipON)
 GO(__openat64_2, iFipO)
-GO(open_by_handle_at, iFipO)
+GO(open_by_handle_at, iFipi)
 //GO(__open_catalog, 
 GOW(opendir, pFp)
 GO(openlog, vFpii)
@@ -1303,9 +1308,9 @@ DATA(opterr, sizeof(int))
 DATA(optind, sizeof(int))
 DATA(optopt, sizeof(int))
 GO(__overflow, iFpi)
-//GO(parse_printf_format, 
+GO(parse_printf_format, LFpLp)
 //GO(passwd2des, // Deprecated
-GOW(pathconf, lFpi)
+GO(pathconf, lFpi)
 GOW(pause, iFv)
 //GO(__pause_nocancel, 
 GO(pclose, iFp)
@@ -1313,7 +1318,7 @@ GO(perror, vFp)
 GOW(personality, iFL)
 GO(__pipe, iFp)
 GOW(pipe, iFp)
-GOW(pipe2, iFpO)
+GO(pipe2, iFpi)
 //GO(pivot_root, // Deprecated
 GO(pkey_alloc, iFuu)
 GO(pkey_free, iFi)
@@ -1326,23 +1331,23 @@ GO(pkey_set, iFiu)
 GO(pmap_set, iFLLiW)
 GO(pmap_unset, iFLL)
 GO(__poll, iFpLi)
-GOW(poll, iFpLi)
+GO(poll, iFpLi)
 GO(__poll_chk, iFpuiL)
 GO(popen, pFpp)
 GO(posix_fadvise, iFilli)
-GOW(posix_fadvise64, iFilli)
+GO(posix_fadvise64, iFilli)
 GO(posix_fallocate, iFill)
 GO(posix_fallocate64, iFill)
 //GO(__posix_getopt, 
 GO(posix_madvise, iFpLi)
-GOW(posix_memalign, iFpLL)
+GO(posix_memalign, iFpLL)
 GOW(posix_openpt, iFi)
 GOM(posix_spawn, iFEpppppp)
 GOW(posix_spawnattr_destroy, iFp)
 GO(posix_spawnattr_getflags, iFpp)
 GO(posix_spawnattr_getpgroup, iFpp)
-//GO(posix_spawnattr_getschedparam, iF!!)
-//GO(posix_spawnattr_getschedpolicy, iF!p)
+GO(posix_spawnattr_getschedparam, iFpp)
+GO(posix_spawnattr_getschedpolicy, iFpp)
 GO(posix_spawnattr_getsigdefault, iFpp)
 GO(posix_spawnattr_getsigmask, iFpp)
 GOW(posix_spawnattr_init, iFp)
@@ -1361,11 +1366,11 @@ GOW(posix_spawn_file_actions_destroy, iFp)
 GOW(posix_spawn_file_actions_init, iFp)
 GOM(posix_spawnp, iFEpppppp)
 GO(ppoll, iFpLpp)
-//GO(__ppoll_chk, 
+GO(__ppoll_chk, iFpuppL)
 GOWM(prctl, iFEiLLLL)
-GOW(pread, lFipLl)
+GO(pread, IFipUI)
 GOW(__pread64, lFipLI)
-GOW(pread64, lFipLl)
+GO(pread64, lFipLl)
 //GO(__pread64_chk, 
 GO(__pread_chk, lFipLlL)
 GO(preadv, lFipil)
@@ -1381,45 +1386,140 @@ GO(prlimit, iFiupp)
 GOW(prlimit64, iFiupp)
 GO(process_vm_readv, lFipLpLL)
 GO(process_vm_writev, lFipLpLL)
-GOW(profil, iFpLLu)
+GO(profil, iFpLLu)
 //GO(__profile_frequency, 
 DATA(__progname, sizeof(void*))
 DATA(__progname_full, sizeof(void))
 GOW(pselect, iFippppp)
-//GO(psiginfo, vF!p)
+GO(psiginfo, vFpp)
 GO(psignal, vFip)
-//GO(pthread_attr_destroy, 
-//GO(pthread_attr_getdetachstate, 
-//GO(pthread_attr_getinheritsched, 
-//GO(pthread_attr_getschedparam, 
-//GO(pthread_attr_getschedpolicy, 
-//GO(pthread_attr_getscope, 
-//GO(pthread_attr_init, 
-//GO(pthread_attr_setdetachstate, 
-//GO(pthread_attr_setinheritsched, 
-//GO(pthread_attr_setschedparam, 
-//GO(pthread_attr_setschedpolicy, 
-//GO(pthread_attr_setscope, 
-//GO(pthread_condattr_destroy, 
-//GO(pthread_condattr_init, 
-//GO(pthread_cond_broadcast, 
-//GO(pthread_cond_destroy, 
-//GO(pthread_cond_init, 
-//GO(pthread_cond_signal, 
-//GO(pthread_cond_timedwait, 
-//GO(pthread_cond_wait, 
-//GO(pthread_equal, 
-//GO(pthread_exit, 
-//GO(pthread_getschedparam, 
-//GO(pthread_mutex_destroy, 
-//GO(pthread_mutex_init, 
-//GO(pthread_mutex_lock, 
-//GO(pthread_mutex_unlock, 
-//GO(pthread_self, 
-//GO(pthread_setcancelstate, 
-//GO(pthread_setcanceltype, 
-//GO(pthread_setschedparam, 
-GO(ptrace, lFuipp)
+GO(__sF, vFip)
+GO(__assert2, vFip)
+
+GOM(pthread_atfork, iFEppp)
+GOM(pthread_attr_destroy, iFEp)
+GOM(pthread_attr_getdetachstate, iFEpp)
+GOM(pthread_attr_getguardsize, iFEpp)
+GOM(pthread_attr_getinheritsched, iFEpp)
+GOM(pthread_attr_getschedparam, iFEpp)
+GOM(pthread_attr_getschedpolicy, iFEpp)
+GOM(pthread_attr_getscope, iFEpp)
+GOM(pthread_attr_getstack, iFEppp)
+GOM(pthread_attr_getstackaddr, iFEpp)
+GOM(pthread_attr_getstacksize, iFEpp)
+GOM(pthread_attr_init, iFEp)
+GOM(pthread_attr_setaffinity_np, iFEpLp)
+GOM(pthread_attr_setdetachstate, iFEpi)
+GOM(pthread_attr_setguardsize, iFEpL)
+GOM(pthread_attr_setinheritsched, iFEpi)
+GOM(pthread_attr_setschedparam, iFEpp)
+GOM(pthread_attr_setschedpolicy, iFEpi)
+GOM(pthread_attr_setscope, iFEpi)
+GOM(pthread_attr_setstackaddr, iFEpp)
+GOM(pthread_attr_setstack, iFEppL)
+GOM(pthread_attr_setstacksize, iFEpL)
+GOM(pthread_barrierattr_destroy, iFEp)
+GOM(pthread_barrierattr_getpshared, iFEpp)
+GOM(pthread_barrierattr_init, iFEp)
+GOM(pthread_barrierattr_setpshared, iFEpi)
+GO(pthread_barrier_destroy, iFp)
+GOM(pthread_barrier_init, iFEppu)
+GO(pthread_barrier_wait, iFp)
+GO(pthread_cancel, iFL)
+GOM(pthread_condattr_destroy, iFEp)
+GOM(pthread_condattr_getclock, iFEpp)
+GOM(pthread_condattr_getpshared, iFEpp)
+GOM(pthread_condattr_init, iFEp)
+GOM(pthread_condattr_setclock, iFEpi)
+GOM(pthread_condattr_setpshared, iFEpi)
+GOM(pthread_cond_broadcast, iFEp)
+GOM(pthread_cond_destroy, iFEp)
+GOM(pthread_cond_init, iFEpp)
+GO(pthread_cond_signal, iFp)
+GOM(pthread_cond_timedwait, iFEppp)
+GOM(pthread_cond_wait, iFEpp)
+GOM(pthread_create, iFEpppp)
+GOM(pthread_cond_clockwait, iFEppip)
+GO(pthread_detach, iFL)
+GO(pthread_equal, iFLL)
+GO(pthread_exit, vFp)
+GOM(pthread_getaffinity_np, iFEpLp)
+GOM(pthread_getattr_np, iFELp)
+GOM(pthread_getattr_default_np, iFEp)
+GOM(pthread_setattr_default_np, iFEp)
+GO(pthread_getcpuclockid, iFLp)
+GO(pthread_getschedparam, iFLpp)
+GO(pthread_getspecific, pFL)
+GO(pthread_getname_np, iFppL)
+GO(pthread_join, iFLp)
+GOM(pthread_key_create, iFEpp)
+GO(pthread_key_delete, iFL)
+GO2(pthread_kill@GLIBC_2.2.5, iFEpi, my_pthread_kill_old)
+GOM(pthread_kill, iFEpi)
+GO(pthread_kill_other_threads_np, vFv)
+GOM(pthread_mutexattr_destroy, iFEp)
+GOM(pthread_mutexattr_getkind_np, iFEpp)
+GOM(pthread_mutexattr_getprotocol, iFEpp)
+GOM(pthread_mutexattr_getrobust, iFEpp)
+GOM(pthread_mutexattr_gettype, iFEpp)
+GOM(pthread_mutexattr_init, iFEp)
+GOM(pthread_mutexattr_setkind_np, iFEpi)
+GOM(pthread_mutexattr_setprotocol, iFEpi)
+GOM(pthread_mutexattr_setpshared, iFEpi)
+GOM(pthread_mutexattr_setrobust, iFEpi)
+GOM(pthread_mutexattr_settype, iFEpi)
+GO(pthread_mutex_consistent, iFp)
+GO(pthread_mutex_destroy, iFp)
+// phtread_mutex_t is 40 bytes on x86_64, but 48bytes on ARM64
+GOM(pthread_mutex_init, iFpp)
+GO(pthread_mutex_lock, iFp)
+GO(pthread_mutex_timedlock, iFpp)
+GO(pthread_mutex_trylock, iFp)
+GO(pthread_mutex_unlock, iFp)
+GOM(pthread_once, iFEpp)
+GO(pthread_rwlockattr_destroy, vFp)
+GO(pthread_rwlockattr_getkind_np, iFpp)
+GO(pthread_rwlockattr_init, iFp)
+GO(pthread_rwlockattr_setkind_np, iFpi)
+GO(pthread_rwlock_destroy, iFp)
+GO(pthread_rwlock_init, iFpp)
+GO(pthread_rwlock_rdlock, iFp)
+GO(pthread_rwlock_tryrdlock, iFp)
+GO(pthread_rwlock_trywrlock, iFp)
+GO(pthread_rwlock_unlock, iFp)
+GO(pthread_rwlock_wrlock, iFp)
+GO(pthread_self, LFv)
+GOM(pthread_setaffinity_np, iFEpLp)
+GO(pthread_setcancelstate, iFip)
+GO(pthread_setcanceltype, iFip)
+GO(pthread_setconcurrency, iFi)
+GO(pthread_setname_np, iFpp)
+GO(pthread_setschedparam, iFLip)
+GO(pthread_setschedprio, iFpi)
+GO(pthread_setspecific, iFLp)
+GO(pthread_sigmask, iFipp)
+GO(pthread_spin_destroy, iFp)
+GO(pthread_spin_init, iFpi)
+GO(pthread_spin_lock, iFp)
+GO(pthread_spin_trylock, iFp)
+GO(pthread_spin_unlock, iFp)
+GO(pthread_testcancel, vFv)
+GO(pthread_timedjoin_np, iFppp)
+GO(pthread_tryjoin_np, iFpp)
+GO(pthread_yield, iFv)
+GO(sem_close, iFp)
+GO(sem_clockwait, iFppp)
+GO(sem_destroy, iFp)
+GO(sem_getvalue, iFpp)
+GO(sem_init, iFpiu)
+GO(sem_open, pFpOM)
+GO(sem_post, iFp)
+GO(sem_timedwait, iFpp)
+GO(sem_trywait, iFp)
+GO(sem_unlink, iFp)
+GO(sem_wait, iFp)
+
+GOM(ptrace, lFEuipp)
 GO(ptsname, pFi)
 GOW(ptsname_r, iFipL)
 //GO(__ptsname_r_chk, 
@@ -1438,14 +1538,14 @@ GO(putspent, iFpp)
 GOW(pututline, pFp)
 GO(pututxline, pFp)
 GO(putw, iFip)
-GO(putwc, uFip)
-GO(putwchar, uFi)
-GO(putwchar_unlocked, uFi)
-GO(putwc_unlocked, uFip)
-GOW(pvalloc, pFL)
-GOW(pwrite, lFipLl)
+GO(putwc, uFup)
+GO(putwchar, uFu)
+GO(putwchar_unlocked, uFu)
+GO(putwc_unlocked, uFup)
+GO(pvalloc, pFL)
+GO(pwrite, IFipUI)
 //GOW(__pwrite64, 
-GOW(pwrite64, lFipLl)
+GO(pwrite64, lFipLl)
 GO(pwritev, lFipil)
 GO(pwritev2, lFipili)
 GO(pwritev64, lFipil)
@@ -1453,7 +1553,7 @@ GO(pwritev64v2, lFipili)
 GO(qecvt, pFDipp)
 GOW(qecvt_r, iFDipppL)
 GO(qfcvt, pFDipp)
-GOW(qfcvt_r, iFDipppL)
+GO(qfcvt_r, iFDipppL)
 GO(qgcvt, pFDip)
 GOM(qsort, vFEpLLp)
 GOWM(qsort_r, vFEpLLpp)
@@ -1472,7 +1572,7 @@ GO(rcmd_af, iFpWppppW)
 //DATAB(__rcmd_errstr, 
 GOW(__read, lFipL)
 GO(read, lFipL)
-GOW(readahead, lFilL)
+GO(readahead, lFilL)
 GO(__read_chk, lFipLL)
 GOW(readdir, pFp) // struct dirent is 280 bytes on x86_64 and ARM64
 GOW(readdir64, pFp)
@@ -1485,7 +1585,7 @@ GOM(readlinkat, iFEippL)
 //GO(__read_nocancel, 
 GOW(readv, lFipi)
 GO(realloc, pFpL)
-GOW(reallocarray, pFpLL)
+GO(reallocarray, pFpLL)
 GOM(realpath, pFEpp)
 GO2(__realpath_chk, pFEppv, my_realpath)
 GO(reboot, iFi)
@@ -1493,12 +1593,12 @@ GO(reboot, iFi)
 GOW(re_compile_fastmap, iFp)
 GOW(re_compile_pattern, pFpLp)
 GOW(__recv, lFipLi)
-GOW(recv, lFipLi)
+GO(recv, lFipLi)
 GO(__recv_chk, lFipLLi)
 GOW(recvfrom, lFipLipp)
 //GO(__recvfrom_chk, 
 GO(recvmmsg, iFipuip)
-GOW(recvmsg, lFipi)
+GO(recvmsg, lFipi)
 //GOW(re_exec, // Deprecated
 GOW(regcomp, iFppi)
 GOW(regerror, LFippL)
@@ -1510,7 +1610,7 @@ GOW(register_printf_modifier, iFp)
 GOWM(register_printf_specifier, iFEipp)
 GOWM(register_printf_type, iFEp)
 //GO(registerrpc, // Deprecated?
-GOW(remap_file_pages, iFpLiLi)
+GO(remap_file_pages, iFpLiLi)
 //GOW(re_match, iFppii!)
 //GOW(re_match_2, iFppipii!i)
 //DATA(re_max_failures, 
@@ -1560,7 +1660,7 @@ GO(ruserok, iFpipp)
 GO(ruserok_af, iFpippW)
 //GO(ruserpass, 
 GO(__sbrk, pFl)
-GOW(sbrk, pFl)
+GO(sbrk, pFl)
 GOW(scalbn, dFdi)
 GOW(scalbnf, fFfi)
 GOW(scalbnl, DFDi)
@@ -1601,16 +1701,16 @@ GO(semget, iFiii)
 GO(semop, iFipL)
 GO(semtimedop, iFipLp)
 GO(__send, lFipLi)
-GOW(send, lFipLi)
+GO(send, lFipLi)
 GO(sendfile, lFiipL)
 GOW(sendfile64, lFiipL)
 GO(__sendmmsg, iFipui)
 GOW(sendmmsg, iFipui)
-GOW(sendmsg, lFipi)
+GO(sendmsg, lFipi)
 GOW(sendto, lFipLipu)
 GO(setaliasent, vFv)
 GO(setbuf, vFpp)
-GOW(setbuffer, vFppL)
+GO(setbuffer, vFppL)
 GOWM(setcontext, iFEp)
 GO(setdomainname, iFpL)
 GO(setegid, iFu)
@@ -1649,7 +1749,7 @@ GOW(setregid, iFuu)
 GOW(setresgid, iFuuu)
 GOW(setresuid, iFuuu)
 GOW(setreuid, iFuu)
-GOW(setrlimit, iFup)
+GOWM(setrlimit, iFEup)
 GOW(setrlimit64, iFup)
 GO(setrpcent, vFi)
 GO(setservent, vFi)
@@ -1666,7 +1766,7 @@ GOW(setuid, iFu)
 GO(setusershell, vFv)
 GOW(setutent, vFv)
 GO(setutxent, vFv)
-GOW(setvbuf, iFppiL)
+GO(setvbuf, iFppiL)
 GO(setxattr, iFpppLi)
 //GO(sgetsgent, 
 //GOW(sgetsgent_r, 
@@ -1692,7 +1792,7 @@ GO(siggetmask, iFv)
 GO(sighold, iFi)
 GO(sigignore, iFi)
 GO(siginterrupt, iFii)
-//GO(sigisemptyset, iF!)
+GO(sigisemptyset, iFp)
 //GO(__sigismember, 
 GO(sigismember, iFpi)
 GOM(siglongjmp, vFEpi)
@@ -1701,7 +1801,7 @@ GO(signalfd, iFipi)
 GO(__signbit, iFd)
 GO(__signbitf, iFf)
 GO(__signbitl, iFD)
-//GO(sigorset, iF!!!)
+GO(sigorset, iFppp)
 //GO(__sigpause, 
 //GOW(sigpause, // Deprecated
 GO(sigpending, iFp)
@@ -1713,7 +1813,7 @@ GOM(sigset, pFEip)
 GOM(__sigsetjmp, iFEpi)
 GOM(sigsetjmp, iFEpi)
 GOW(sigsetmask, iFi)
-//GO(sigstack, iF!!)
+GO(sigstack, iFpp)
 GO(__sigsuspend, iFp)
 GOW(sigsuspend, iFp)
 GO(__sigtimedwait, iFppp)
@@ -1735,7 +1835,7 @@ GOM(__sprintf_chk, iFEpilpV)
 //GOW(sprofil, 
 GOW(srand, vFu)
 GO(srand48, vFl)
-GOW(srand48_r, iFlp)
+GO(srand48_r, iFlp)
 GOW(srandom, vFu)
 GOW(srandom_r, iFup)
 GOM(sscanf, iFEppV)
@@ -1760,7 +1860,7 @@ GO(stpcpy, pFpp)
 GO(__stpcpy_chk, pFppL)
 //GO(__stpcpy_small, 
 GO(__stpncpy, pFppL)
-GOW(stpncpy, pFppL)
+GO(stpncpy, pFppL)
 GO(__stpncpy_chk, pFppLL)
 GO(__strcasecmp, iFpp)
 GO(strcasecmp, iFpp)
@@ -1788,17 +1888,17 @@ GOW(strdup, pFp)
 GO(strerror, pFi)
 GO(strerror_l, pFip)
 GO(__strerror_r, pFipL)
-GOW(strerror_r, pFipL)
+GO(strerror_r, pFipL)
 //GO(strfmon, lFpLpV)
 //GO(__strfmon_l, 
 //GOW(strfmon_l, lFpLppV)
 GO(strfromd, iFpLpd)
 GO(strfromf, iFpLpf)
-//GO(strfromf128, 
-GOW(strfromf32, iFpLpf)
-GOW(strfromf32x, iFpLpd)
-GOW(strfromf64, iFpLpd)
-GOW(strfromf64x, iFpLpD)
+GO(strfromf128, iFpLpD)
+GO(strfromf32, iFpLpf)
+GO(strfromf32x, iFpLpd)
+GO(strfromf64, iFpLpd)
+GO(strfromf64x, iFpLpD)
 GO(strfroml, iFpLpD)
 GO(strfry, pFp)
 GO(strftime, LFpLpp)
@@ -1814,7 +1914,7 @@ GO(strncmp, iFppL)
 GO(strncpy, pFppL)
 GO(__strncpy_chk, pFppLL)
 GO(__strndup, pFpL)
-GOW(strndup, pFpL)
+GO(strndup, pFpL)
 GO(strnlen, LFpL)
 GO(strpbrk, pFpp)
 //GO(__strpbrk_c2, 
@@ -1839,7 +1939,7 @@ GO(__strtod_l, dFppp)
 GOW(strtod_l, dFppp)
 //GO(__strtod_nan, 
 GO(strtof, fFpp)
-//GO(strtof128, 
+GO(strtof128, DFpp)
 //GO(__strtof128_internal, 
 //GOW(strtof128_l, 
 //GO(__strtof128_nan, 
@@ -1860,7 +1960,7 @@ GO(strtok, pFpp)
 GO(__strtok_r, pFppp)
 GOW(strtok_r, pFppp)
 //GO(__strtok_r_1c, 
-GOW(strtol, lFppi)
+GO(strtol, lFppi)
 GO(strtold, DFpp)
 GO(__strtold_internal, DFppi)
 GO(__strtold_l, DFppp)
@@ -1868,16 +1968,16 @@ GOW(strtold_l, DFppp)
 //GO(__strtold_nan, 
 GO(__strtol_internal, lFppii)
 GO(__strtol_l, lFppi)
-GOW(strtol_l, lFppip)
+GO(strtol_l, lFppip)
 GOW(strtoll, IFppi)
 GO(__strtoll_internal, IFppii)
 GOW(__strtoll_l, IFppip)
 GOW(strtoll_l, IFppip)
 GOW(strtoq, IFppi)
-GOW(strtoul, LFppi)
+GO(strtoul, LFppi)
 GO(__strtoul_internal, LFppii)
 //GO(__strtoul_l, 
-GOW(strtoul_l, LFppip)
+GO(strtoul_l, LFppip)
 GOW(strtoull, UFppi)
 GO(__strtoull_internal, UFppii)
 GOW(__strtoull_l, LFppip)
@@ -1888,7 +1988,7 @@ GOW(strtouq, UFppi)
 GOW(strverscmp, iFpp)
 GO(strxfrm, LFppL)
 GO(__strxfrm_l, LFppLL)
-GOW(strxfrm_l, LFppLp)
+GO(strxfrm_l, LFppLp)
 //GO(stty, // Deprecated
 //DATAB(svcauthdes_stats, 
 //GO(svcerr_auth, vF!u)
@@ -1931,8 +2031,8 @@ GO(sync, vFv)
 GO(sync_file_range, iFillu)
 GO(syncfs, iFi)
 GOM(syscall, lFEv)
-GO(__sysconf, lFi)
-GOW(sysconf, lFi)
+GOM(__sysconf, lFEi)
+GOM(sysconf, IFEi)
 GO(__sysctl, iFpipppL)
 GOW(sysctl, iFpipppL) // Deprecated
 DATA(_sys_errlist, sizeof(void*))
@@ -1956,14 +2056,14 @@ GO(tcsendbreak, iFii)
 GO(tcsetattr, iFiip)
 GO(tcsetpgrp, iFii)
 //GO(__tdelete, 
-//GOW(tdelete, pFpp@)
+GOWM(tdelete, pFEppp)
 GOWM(tdestroy, vFEpp)
 GO(tee, lFiiLu)
 GO(telldir, lFp)
 GO(tempnam, pFpp)
 GOW(textdomain, pFp)
 //GO(__tfind, 
-//GOW(tfind, pFpp@)
+GOWM(tfind, pFEppp)
 GOW(tgkill, iFiii)
 //GO(thrd_current, 
 //GO(thrd_equal, 
@@ -2004,12 +2104,12 @@ GOW(towupper, uFu)
 GO(__towupper_l, iFip)
 GOW(towupper_l, uFup)
 //GO(tr_break, 
-GOW(truncate, iFpl)
-GOW(truncate64, iFpl)
+GO(truncate, iFpI)
+GO(truncate64, iFpl)
 //GO(__tsearch, 
 GOWM(tsearch, pFEppp)
 GO(ttyname, pFi)
-GOW(ttyname_r, iFipL)
+GO(ttyname_r, iFipL)
 //GO(__ttyname_r_chk, 
 GO(ttyslot, iFv)
 //GO(__twalk, 
@@ -2047,11 +2147,11 @@ GO(utimensat, iFippi)
 GOW(utimes, iFpp)
 GOW(utmpname, iFp)
 GO(utmpxname, iFp)
-GOW(valloc, pFL)
+GO(valloc, pFL)
 GOWM(vasprintf, iFEppA)
 GOM(__vasprintf_chk, iFEpipp)
-//GOW(vdprintf, iFipA)
-//GO(__vdprintf_chk, 
+GOM(vdprintf, iFEipA)
+GOM(__vdprintf_chk, iFEiipA)
 //GO(verr, vFipA)
 //GO(verrx, vFipA)
 GOW(versionsort, iFpp)
@@ -2073,7 +2173,7 @@ GOM(__vprintf_chk, iFEvpp)
 //GOW(vscanf, iFpA)
 GOWM(__vsnprintf, iFEpLpA)
 GOWM(vsnprintf, iFEpLpA)
-GOM(__vsnprintf_chk, iFEpLvvpp)
+GOM(__vsnprintf_chk, iFEpLiLpp)
 GOWM(vsprintf, iFEppA)
 GOM(__vsprintf_chk, iFEpvvppp)
 GOWM(__vsscanf, iFEppp)
@@ -2086,7 +2186,7 @@ GOM(__vsyslog_chk, vFEiipA)
 //GO(vtimes, // Deprecated
 //GO(vwarn, vFpA)
 //GO(vwarnx, vFpA)
-//GO(vwprintf, iFpA)
+GOM(vwprintf, iFEpA)
 //GO(__vwprintf_chk, 
 //GO(vwscanf, iFpA)
 GOW(__wait, iFp)
@@ -2100,17 +2200,17 @@ GOM(warn, vFEpV)
 GOM(warnx, vFEpV)
 GOW(wcpcpy, pFpp)
 //GO(__wcpcpy_chk, 
-GOW(wcpncpy, pFppL)
+GO(wcpncpy, pFppL)
 //GO(__wcpncpy_chk, 
-GOW(wcrtomb, LFpip)
+GO(wcrtomb, LFpup)
 //GO(__wcrtomb_chk, 
 GOW(wcscasecmp, iFpp)
 GO(__wcscasecmp_l, iFppp)
 GOW(wcscasecmp_l, iFppp)
 GOW(wcscat, pFpp)
 GO(__wcscat_chk, pFppL)
-GO(wcschr, pFpi)
-GOW(wcschrnul, pFpi)
+GO(wcschr, pFpu)
+GO(wcschrnul, pFpu)
 GO(wcscmp, iFpp)
 GOW(wcscoll, iFpp)
 GO(__wcscoll_l, iFppp)
@@ -2121,9 +2221,9 @@ GO(wcscspn, LFpp)
 GO(wcsdup, pFp)
 GO(wcsftime, LFpLpp)
 GO(__wcsftime_l, LFpLppp)
-GOW(wcsftime_l, LFpLppp)
+GO(wcsftime_l, LFpLppp)
 GO(wcslen, LFp)
-GOW(wcsncasecmp, iFppL)
+GO(wcsncasecmp, iFppL)
 //GO(__wcsncasecmp_l, 
 //GOW(wcsncasecmp_l, iFppL!)
 GO(wcsncat, pFppL)
@@ -2132,11 +2232,11 @@ GO(wcsncmp, iFppL)
 GOW(wcsncpy, pFppL)
 GO(__wcsncpy_chk, pFppLL)
 GO(wcsnlen, LFpL)
-GOW(wcsnrtombs, LFppLLp)
+GO(wcsnrtombs, LFppLLp)
 //GO(__wcsnrtombs_chk, 
 GO(wcspbrk, pFpp)
-GO(wcsrchr, pFpi)
-GOW(wcsrtombs, LFppLp)
+GO(wcsrchr, pFpu)
+GO(wcsrtombs, LFppLp)
 //GO(__wcsrtombs_chk, 
 GO(wcsspn, LFpp)
 GO(wcsstr, pFpp)
@@ -2145,7 +2245,7 @@ GO(wcstod, dFpp)
 //GO(__wcstod_l, 
 //GOW(wcstod_l, dFpp!)
 GO(wcstof, fFpp)
-//GO(wcstof128, 
+GO(wcstof128, DFpp)
 //GO(__wcstof128_internal, 
 //GOW(wcstof128_l, 
 GOW(wcstof32, fFpp)
@@ -2161,7 +2261,7 @@ GOW(wcstof64x, DFpp)
 //GOW(wcstof_l, fFpp!)
 GO(wcstoimax, lFppi)
 GO(wcstok, pFppp)
-GOW(wcstol, lFppi)
+GO(wcstol, lFppi)
 GO(wcstold, DFpp)
 //GO(__wcstold_internal, 
 //GO(__wcstold_l, 
@@ -2176,7 +2276,7 @@ GOW(wcstoll, IFppi)
 GO(wcstombs, LFppL)
 //GO(__wcstombs_chk, 
 GOW(wcstoq, IFppi)
-GOW(wcstoul, LFppi)
+GO(wcstoul, LFppi)
 //GO(__wcstoul_internal, 
 //GO(__wcstoul_l, 
 //GOW(wcstoul_l, LFppi!)
@@ -2190,9 +2290,9 @@ GOW(wcswcs, pFpp)
 GO(wcswidth, iFpL)
 GO(wcsxfrm, LFppL)
 GO(__wcsxfrm_l, LFppLL)
-GOW(wcsxfrm_l, LFppLp)
+GO(wcsxfrm_l, LFppLp)
 GO(wctob, iFu)
-GO(wctomb, iFpi)
+GO(wctomb, iFpu)
 GO(__wctomb_chk, iFpuL)
 GOW(wctrans, pFp)
 //GO(__wctrans_l, 
@@ -2200,24 +2300,24 @@ GOW(wctrans_l, pFpp)
 GOW(wctype, LFp)
 GOW(wctype_l, LFpp)
 GO(__wctype_l, LFpp)
-GO(wcwidth, iFi)
-GO(wmemchr, pFpiL)
+GO(wcwidth, iFu)
+GO(wmemchr, pFpuL)
 GO(wmemcmp, iFppL)
 GOW(wmemcpy, pFppL)
 GO(__wmemcpy_chk, pFppLL)
-GOW(wmemmove, pFppL)
-//GO(__wmemmove_chk, 
+GO(wmemmove, pFppL)
+GO(__wmemmove_chk, pFppLL)
 GOW(wmempcpy, pFppL)
 //GO(__wmempcpy_chk, 
-GO(wmemset, pFpiL)
-//GO(__wmemset_chk, 
+GO(wmemset, pFpuL)
+GO(__wmemset_chk, pFpuLL)
 GO(wordexp, iFppi)
 GO(wordfree, vFp)
 //GO(__woverflow, 
 GOM(wprintf, iFEpV)
 GOM(__wprintf_chk, lFEipV)
 GOW(__write, lFipL)
-GOW(write, lFipL)
+GO(write, lFipL)
 //GO(__write_nocancel, 
 GOW(writev, lFipi)
 //GO(wscanf, iFpV)
@@ -2325,3 +2425,12 @@ DATAM(__libc_single_threaded, 1)
 
 GO(iconvctl, iFlip)
 GO(dummy__ZnwmSt11align_val_tRKSt9nothrow_t, pFLLp) // for mallochook.c
+
+#ifdef ANDROID
+GOM(__libc_init, vFEpppp)
+GO(__errno, pFv)
+#else
+// Those symbols don't exist in non-Android builds
+//GOM(__libc_init,
+//GO(__errno,
+#endif
