@@ -2,4 +2,4 @@
 #error meh!
 #endif
 
-//GO(vdp_device_create_x11, iFpipp) // last is a pointer to a VdpGetProcAddress function
\ No newline at end of file
+GOM(vdp_device_create_x11, iFpipp) // last is a pointer to a VdpGetProcAddress function
\ No newline at end of file
