@@ -17,7 +17,12 @@
 #include "box64context.h"
 #include "emu/x64emu_private.h"
 
-const char* libxcbName = "libxcb.so.1";
+#ifdef ANDROID
+	const char* libxcbName = "libxcb.so";
+#else
+	const char* libxcbName = "libxcb.so.1";
+#endif
+
 #define LIBNAME libxcb
 
 #include "wrappedlib_init.h"
