@@ -148,7 +148,7 @@ GO(xcb_create_gc_checked, pFpuuup)
 GO(xcb_create_glyph_cursor, pFpuuuWWWWWWWW)
 //GO(xcb_create_glyph_cursor_checked, 
 GO(xcb_create_pixmap, pFpCuuWW)
-//GO(xcb_create_pixmap_checked, 
+GO(xcb_create_pixmap_checked, pFpCuuWW)
 GO(xcb_create_window, pFpCuuwwWWWWuup)
 //GO(xcb_create_window_aux, 
 //GO(xcb_create_window_aux_checked, 
@@ -207,7 +207,7 @@ GO(xcb_free_cursor, pFpp)
 GO(xcb_free_gc, pFpu)
 //GO(xcb_free_gc_checked, 
 GO(xcb_free_pixmap, pFpu)
-//GO(xcb_free_pixmap_checked, 
+GO(xcb_free_pixmap_checked, pFpu)
 //GO(xcb_gcontext_end, 
 //GO(xcb_gcontext_next, 
 GO(xcb_generate_id, uFp)
@@ -460,9 +460,9 @@ GO(xcb_poly_segment, pFpuuup)
 //GO(xcb_poly_text_8_sizeof, 
 GO(xcb_popcount, iFu)
 GO(xcb_prefetch_extension_data, vFpp)
-//GO(xcb_prefetch_maximum_request_length, 
+GO(xcb_prefetch_maximum_request_length, vFp)
 GO(xcb_put_image, pFpCuuWWwwCCup)
-//GO(xcb_put_image_checked, 
+GO(xcb_put_image_checked, pFpCuuWWwwCCup)
 //GO(xcb_put_image_data, 
 //GO(xcb_put_image_data_end, 
 //GO(xcb_put_image_data_length, 
@@ -491,9 +491,9 @@ GO(xcb_put_image, pFpCuuWWwwCCup)
 //GO(xcb_query_font_reply, 
 //GO(xcb_query_font_sizeof, 
 //GO(xcb_query_font_unchecked, 
-//GO(xcb_query_keymap, 
-//GO(xcb_query_keymap_reply, 
-//GO(xcb_query_keymap_unchecked, 
+GO(xcb_query_keymap, pFpp)
+GO(xcb_query_keymap_reply, pFpup)
+GO(xcb_query_keymap_unchecked, pFpp)
 GO(xcb_query_pointer, pFpu)
 GO(xcb_query_pointer_reply, pFpup)
 //GO(xcb_query_pointer_unchecked, 
@@ -533,7 +533,7 @@ GO(xcb_screen_next, vFp)
 //GO(xcb_segment_next, 
 GO(xcb_send_event, pFpCpup)
 GO(xcb_send_event_checked, pFpCpup)
-//GO(xcb_send_fd, 
+GO(xcb_send_fd, vFpi)
 GO(xcb_send_request, uFpipp)
 GO(xcb_send_request64, UFpipp)
 GO(xcb_send_request_with_fds, uFpippup)
@@ -622,7 +622,7 @@ GO(xcb_setup_roots_length, iFp)
 //GO(xcb_str_name_end, 
 //GO(xcb_str_name_length, 
 //GO(xcb_str_next, 
-//GO(xcb_str_sizeof, 
+GO(xcb_str_sizeof, iFp)
 //GO(xcb_sumof, 
 //GO(xcb_take_socket, 
 //GO(xcb_timecoord_end, 
