@@ -17,7 +17,12 @@
 #include "box64context.h"
 #include "emu/x64emu_private.h"
 
-const char* libxcbglxName = "libxcb-glx.so.0";
+#ifdef ANDROID
+	const char* libxcbglxName = "libxcb-glx.so";
+#else
+	const char* libxcbglxName = "libxcb-glx.so.0";
+#endif
+
 #define LIBNAME libxcbglx
 
 #include "wrappedlib_init.h"
