@@ -11,11 +11,21 @@
 #include "librarian/library_private.h"
 #include "x64emu.h"
 
-const char* libxmuName = "libXmu.so.6";
+#ifdef ANDROID
+    const char* libxmuName = "libXmu.so";
+#else
+    const char* libxmuName = "libXmu.so.6";
+#endif
+
 #define LIBNAME libxmu
 
-#define CUSTOM_INIT \
-    setNeededLibs(lib, 2, "libX11.so.6", "libXext.so.6");
+#ifdef ANDROID
+    #define CUSTOM_INIT \
+        setNeededLibs(lib, 2, "libX11.so", "libXext.so");
+#else
+    #define CUSTOM_INIT \
+        setNeededLibs(lib, 2, "libX11.so.6", "libXext.so.6");
+#endif
 
 #include "wrappedlib_init.h"
 
