@@ -17,11 +17,20 @@
 #include "box64context.h"
 #include "emu/x64emu_private.h"
 
-const char* libxxf86vmName = "libXxf86vm.so.1";
+#ifdef ANDROID
+    const char* libxxf86vmName = "libXxf86vm.so";
+#else
+    const char* libxxf86vmName = "libXxf86vm.so.1";
+#endif
+
 #define LIBNAME libxxf86vm
 
-
-#define CUSTOM_INIT \
-    setNeededLibs(lib, 2, "libX11.so.6", "libXext.so.6");
+#ifdef ANDROID
+    #define CUSTOM_INIT \
+        setNeededLibs(lib, 2, "libX11.so", "libXext.so");
+#else
+    #define CUSTOM_INIT \
+        setNeededLibs(lib, 2, "libX11.so.6", "libXext.so.6");
+#endif
 
 #include "wrappedlib_init.h"
