@@ -24,14 +24,14 @@ GO(SDL_atanf, fFf)
 GO(SDL_atof, dFp)
 GO(SDL_atoi, iFp)
 GO(SDL_AtomicAdd, iFpi)
-GO(SDL_AtomicCAS, iFpii)
-GO(SDL_AtomicCASPtr, iFppp)
+GO(SDL_AtomicCAS, uFpii)
+GO(SDL_AtomicCASPtr, uFppp)
 GO(SDL_AtomicGet, iFp)
 GO(SDL_AtomicGetPtr, pFp)
 GO(SDL_AtomicLock, vFp)
 GO(SDL_AtomicSet, iFpi)
 GO(SDL_AtomicSetPtr, pFpp)
-GO(SDL_AtomicTryLock, iFp)
+GO(SDL_AtomicTryLock, uFp)
 GO(SDL_AtomicUnlock, vFp)
 GO(SDL_AudioInit, iFp)
 GO(SDL_AudioQuit, vFv)
@@ -40,18 +40,18 @@ GO(SDL_AudioStreamClear, vFp)
 GO(SDL_AudioStreamFlush, iFp)
 GO(SDL_AudioStreamGet, iFppi)
 GO(SDL_AudioStreamPut, iFppi)
-GO(SDL_BuildAudioCVT, iFpuuiuui)
+GO(SDL_BuildAudioCVT, iFpWCiWCi)
 GO(SDL_CalculateGammaRamp, vFfp)
-GO(SDL_calloc, pFuu)
-GO(SDL_CaptureMouse, iFi)
+GO(SDL_calloc, pFLL)
+GO(SDL_CaptureMouse, iFu)
 GO(SDL_ceil, dFd)
 GO(SDL_ceilf, fFf)
 GO(SDL_ClearError, vFv)
 GO(SDL_ClearHints, vFv)
 GO(SDL_ClearQueuedAudio, vFu)
 GO(SDL_CloseAudio, vFv)
-GO(SDL_CloseAudioDevice, vFi)
-GO(SDL_ComposeCustomBlendMode, iFiiiiii)
+GO(SDL_CloseAudioDevice, vFu)
+GO(SDL_ComposeCustomBlendMode, uFuuuuuu)
 GO(SDL_CondBroadcast, iFp)
 GO(SDL_CondSignal, iFp)
 GO(SDL_CondWait, iFpp)
@@ -99,8 +99,8 @@ GO(SDL_DisableScreenSaver, vFv)
 GO(SDL_DuplicateSurface, pFp)
 GOM(SDL_DYNAPI_entry, iFEupu)
 GO(SDL_EnableScreenSaver, vFv)
-GO(SDL_EnclosePoints, iFpipp)
-GO(SDL_Error, iFi)
+GO(SDL_EnclosePoints, uFpipp)
+GO(SDL_Error, iFu)
 GO(SDL_EventState, CFui)
 GO(SDL_exp, dFd)
 GO(SDL_expf, fFf)
@@ -130,12 +130,12 @@ GO(SDL_GameControllerEventState, iFi)
 GO(SDL_GameControllerFromInstanceID, pFi)
 GO(SDL_GameControllerFromPlayerIndex, pFi)
 GO(SDL_GameControllerGetAttached, iFp)
-GO(SDL_GameControllerGetAxis, iFpi)
-GO(SDL_GameControllerGetAxisFromString, uFp)
+GO(SDL_GameControllerGetAxis, wFpi)
+GO(SDL_GameControllerGetAxisFromString, iFp)
 GO(SDL_GameControllerGetBindForAxis, HFpi)
 GO(SDL_GameControllerGetBindForButton, HFpi)
-GO(SDL_GameControllerGetButton, uFpi)
-GO(SDL_GameControllerGetButtonFromString, uFp)
+GO(SDL_GameControllerGetButton, CFpi)
+GO(SDL_GameControllerGetButtonFromString, iFp)
 GO(SDL_GameControllerGetJoystick, pFp)
 GO(SDL_GameControllerGetNumTouchpads, iFp)
 GO(SDL_GameControllerGetNumTouchpadFingers, iFpi)
@@ -144,15 +144,15 @@ GO(SDL_GameControllerGetProduct, WFp)
 GO(SDL_GameControllerGetProductVersion, WFp)
 GO(SDL_GameControllerGetSensorData, iFpipi)
 GO(SDL_GameControllerGetSerial, pFp)
-GO(SDL_GameControllerGetStringForAxis, pFp)
-GO(SDL_GameControllerGetStringForButton, pFu)
+GO(SDL_GameControllerGetStringForAxis, pFi)
+GO(SDL_GameControllerGetStringForButton, pFi)
 GO(SDL_GameControllerGetTouchpadFinger, iFpiipppp)
-GO(SDL_GameControllerGetType, iFp)
+GO(SDL_GameControllerGetType, uFp)
 GO(SDL_GameControllerGetVendor, WFp)
-GO(SDL_GameControllerHasButton, iFpi)
+GO(SDL_GameControllerHasButton, uFpi)
 GO(SDL_GameControllerHasLED, iFp)
 GO(SDL_GameControllerHasSensor, iFpi)
-GO(SDL_GameControllerIsSensorEnabled, iFpi)
+GO(SDL_GameControllerIsSensorEnabled, uFpi)
 GO(SDL_GameControllerMapping, pFp)
 GO(SDL_GameControllerMappingForDeviceIndex, pFi)
 GO(SDL_GameControllerMappingForGUID, pFpp)
@@ -165,16 +165,16 @@ GO(SDL_GameControllerRumble, iFpWWu)
 GO(SDL_GameControllerRumbleTriggers, iFpWWu)
 GO(SDL_GameControllerSetLED, iFpCCC)
 GO(SDL_GameControllerSetPlayerIndex, vFpi)
-GO(SDL_GameControllerSetSensorEnabled, iFpii)
-GO(SDL_GameControllerTypeForIndex, iFi)
+GO(SDL_GameControllerSetSensorEnabled, iFpiu)
+GO(SDL_GameControllerTypeForIndex, uFi)
 GO(SDL_GameControllerUpdate, vFv)
 // SDL_GetAssertionHandler
 GO(SDL_GetAssertionReport, pFv)
 GO(SDL_GetAudioDeviceName, pFii)
-GO(SDL_GetAudioDeviceStatus, iFi)
+GO(SDL_GetAudioDeviceStatus, uFu)
 GO(SDL_GetAudioDriver, pFi)
 GO(SDL_GetAudioDeviceSpec, iFiip)
-GO(SDL_GetAudioStatus, iFv)
+GO(SDL_GetAudioStatus, uFv)
 GOM(SDL_GetBasePath, pFEv)
 GO(SDL_GetClipboardText, pFv)
 GO(SDL_GetClipRect, vFpp)
@@ -193,7 +193,7 @@ GO(SDL_GetDisplayBounds, iFip)
 GO(SDL_GetDisplayDPI, iFippp)
 GO(SDL_GetDisplayMode, iFiip)
 GO(SDL_GetDisplayName, pFi)
-GO(SDL_GetDisplayOrientation, iFi)
+GO(SDL_GetDisplayOrientation, uFi)
 GO(SDL_GetDisplayUsableBounds, iFip)
 GO(SDL_getenv, pFp)
 GO(SDL_GetError, pFv)
@@ -201,15 +201,15 @@ GOM(SDL_GetEventFilter, iFEpp)
 GO(SDL_GetGlobalMouseState, uFpp)
 GO(SDL_GetGrabbedWindow, pFv)
 GO(SDL_GetHint, pFp)
-GO(SDL_GetHintBoolean, iFpi)
-GOM(SDL_GetJoystickGUIDInfo, vFGppp)
+GO(SDL_GetHintBoolean, uFpu)
+GOM(SDL_GetJoystickGUIDInfo, vFGpppp)
 GO(SDL_GetKeyboardFocus, pFv)
 GO(SDL_GetKeyboardState, pFp)
-GO(SDL_GetKeyFromName, pFp)
-GO(SDL_GetKeyFromScancode, uFu)
-GO(SDL_GetKeyName, pFu)
-GO(SDL_GetMemoryFunctions, iFpi)
-GO(SDL_GetModState, pFv)
+GO(SDL_GetKeyFromName, iFp)
+GO(SDL_GetKeyFromScancode, iFu)
+GO(SDL_GetKeyName, pFi)
+GO(SDL_GetMemoryFunctions, vFpppp)
+GO(SDL_GetModState, uFv)
 GO(SDL_GetMouseFocus, pFv)
 GO(SDL_GetMouseState, uFpp)
 GO(SDL_GetNumAllocations, iFv)
@@ -222,13 +222,13 @@ GO(SDL_GetNumTouchFingers, iFu)
 GO(SDL_GetNumVideoDisplays, iFv)
 GO(SDL_GetNumVideoDrivers, iFv)
 GO(SDL_GetPerformanceCounter, UFv)
-GO(SDL_GetPerformanceFrequency, UFv)
+GO(SDL_GetPerformanceFrequency, LFv)
 GO(SDL_GetPixelFormatName, pFu)
 GO(SDL_GetPlatform, pFv)
-GO(SDL_GetPowerInfo, iFpp)
+GO(SDL_GetPowerInfo, uFpp)
 GO(SDL_GetPreferredLocales, pFv)
 GO(SDL_GetPrefPath, pFpp)
-GO(SDL_GetQueuedAudioSize, uFi)
+GO(SDL_GetQueuedAudioSize, uFu)
 GO(SDL_GetRelativeMouseMode, iFv)
 GO(SDL_GetRelativeMouseState, uFpp)
 GO(SDL_GetRenderDrawBlendMode, iFpp)
@@ -242,8 +242,8 @@ GO(SDL_GetRevision, pFv)
 GO(SDL_GetRevisionNumber, iFv)
 GO(SDL_GetRGB, vFupppp)
 GO(SDL_GetRGBA, vFuppppp)
-GO(SDL_GetScancodeFromKey, uFu)
-GO(SDL_GetScancodeFromName, iFp)
+GO(SDL_GetScancodeFromKey, uFi)
+GO(SDL_GetScancodeFromName, uFp)
 GO(SDL_GetScancodeName, pFu)
 GO(SDL_GetShapedWindowMode, iFpp)
 GO(SDL_GetSurfaceAlphaMod, iFpp)
@@ -256,9 +256,9 @@ GO(SDL_GetTextureColorMod, iFpppp)
 GO(SDL_GetThreadID, LFp)
 GO(SDL_GetThreadName, pFp)
 GO(SDL_GetTicks, uFv)
-GO(SDL_GetTouchDevice, uFi)
-GO(SDL_GetTouchDeviceType, iFi)
-GO(SDL_GetTouchFinger, pFui)
+GO(SDL_GetTouchDevice, lFi)
+GO(SDL_GetTouchDeviceType, iFl)
+GO(SDL_GetTouchFinger, pFli)
 GO(SDL_GetVersion, vFp)
 GO(SDL_GetVideoDriver, pFi)
 GO(SDL_GetWindowBordersSize, iFppppp)
@@ -280,8 +280,8 @@ GO(SDL_GetWindowSize, vFppp)
 GO(SDL_GetWindowSurface, pFp)
 GO(SDL_GetWindowTitle, pFp)
 GO(SDL_GetWindowWMInfo, iFpp)
-GO(SDL_GetYUVConversionMode, iFv)
-GO(SDL_GetYUVConversionModeForResolution, iFii)
+GO(SDL_GetYUVConversionMode, uFv)
+GO(SDL_GetYUVConversionModeForResolution, uFii)
 GO(SDL_GL_BindTexture, iFppp)
 GO(SDL_GL_CreateContext, pFp)
 GO(SDL_GL_DeleteContext, vFp)
@@ -333,10 +333,10 @@ GOM(SDL_HasAVX, iFv)
 GOM(SDL_HasAVX2, iFv)
 GOM(SDL_HasAVX512F, iFv)
 GO(SDL_HasClipboardText, iFv)
-GO(SDL_HasColorKey, iFv)
-GO(SDL_HasEvent, iFv)
-GO(SDL_HasEvents, iFv)
-GO(SDL_HasIntersection, iFpp)
+GO(SDL_HasColorKey, uFp)
+GO(SDL_HasEvent, uFu)
+GO(SDL_HasEvents, uFuu)
+GO(SDL_HasIntersection, uFpp)
 GOM(SDL_HasMMX, iFv)
 GOM(SDL_HasNEON, iFv)
 GOM(SDL_HasRDTSC, iFv)
@@ -353,15 +353,15 @@ GO(SDL_iconv_open, pFpp)
 GO(SDL_iconv_string, pFpppL)
 GO(SDL_Init, iFu)
 GO(SDL_InitSubSystem, iFu)
-GO(SDL_IntersectRect, iFppp)
-GO(SDL_IntersectRectAndLine, iFppppp)
+GO(SDL_IntersectRect, uFppp)
+GO(SDL_IntersectRectAndLine, uFppppp)
 GO(SDL_isdigit, iFi)
-GO(SDL_IsGameController, iFi)
-GO(SDL_IsScreenKeyboardShown, iFp)
+GO(SDL_IsGameController, uFi)
+GO(SDL_IsScreenKeyboardShown, uFp)
 GO(SDL_IsScreenSaverEnabled, iFv)
-GO(SDL_IsShapedWindow, iFp)
+GO(SDL_IsShapedWindow, uFp)
 GO(SDL_isspace, iFi)
-GO(SDL_IsTablet, iFv)
+GO(SDL_IsTablet, uFv)
 GO(SDL_IsTextInputActive, iFv)
 GO(SDL_itoa, pFipi)
 GO(SDL_JoystickClose, vFp)
@@ -369,8 +369,8 @@ GO(SDL_JoystickCurrentPowerLevel, iFp)
 GO(SDL_JoystickEventState, iFi)
 GO(SDL_JoystickFromInstanceID, pFi)
 GO(SDL_JoystickGetAttached, iFp)
-GO(SDL_JoystickGetAxis, iFpi)
-GO(SDL_JoystickGetAxisInitialState, iFpip)
+GO(SDL_JoystickGetAxis, wFpi)
+GO(SDL_JoystickGetAxisInitialState, uFpip)
 GO(SDL_JoystickGetBall, iFpipp)
 GO(SDL_JoystickGetButton, CFpi)
 GO(SDL_JoystickGetDeviceGUID, HFi)  // return a struct that is "uint8_t data[16]"
@@ -378,7 +378,7 @@ GO(SDL_JoystickGetDeviceInstanceID, iFi)
 GO(SDL_JoystickGetDevicePlayerIndex, iFi)
 GO(SDL_JoystickGetDeviceProduct, WFi)
 GO(SDL_JoystickGetDeviceProductVersion, WFi)
-GO(SDL_JoystickGetDeviceType, iFi)
+GO(SDL_JoystickGetDeviceType, uFi)
 GO(SDL_JoystickGetDeviceVendor, WFi)
 GO(SDL_JoystickGetGUID, HFp)
 GO(SDL_JoystickGetGUIDFromString, HFp)
@@ -387,7 +387,7 @@ GO(SDL_JoystickGetHat, CFpi)
 GO(SDL_JoystickGetPlayerIndex, iFp)
 GO(SDL_JoystickGetProduct, WFp)
 GO(SDL_JoystickGetProductVersion, WFp)
-GO(SDL_JoystickGetType, iFp)
+GO(SDL_JoystickGetType, uFp)
 GO(SDL_JoystickGetVendor, WFp)
 GO(SDL_JoystickInstanceID, iFp)
 GO(SDL_JoystickIsHaptic, iFp)
@@ -400,8 +400,8 @@ GO(SDL_JoystickNumHats, iFp)
 GO(SDL_JoystickOpen, pFi)
 GO(SDL_JoystickRumble, iFpWWu)
 GO(SDL_JoystickUpdate, vFv)
-GO(SDL_LinuxSetThreadPriority, iFIi)
-GO(SDL_lltoa, pFIpi)
+GO(SDL_LinuxSetThreadPriority, iFli)
+GO(SDL_lltoa, pFlpi)
 GOM(SDL_LoadBMP_RW, pFEpi)
 GO(SDL_LoadDollarTemplates, iFip)
 GO(SDL_LoadFile, pFpp)
@@ -424,7 +424,7 @@ GOM(SDL_LogDebug, vFEipV)
 GOM(SDL_LogError, vFEipV)
 GO(SDL_logf, fFf)
 GOM(SDL_LogGetOutputFunction, vFEpp)
-GO(SDL_LogGetPriority, iFi)
+GO(SDL_LogGetPriority, uFi)
 GOM(SDL_LogInfo, vFEipV)
 GO2(SDL_LogMessage, vFiupV, SDL_LogMessageV)
 GO(SDL_LogMessageV, vFiupV)
@@ -436,32 +436,32 @@ GOM(SDL_LogVerbose, vFEipV)
 GOM(SDL_LogWarn, vFEipV)
 GO(SDL_LowerBlit, iFpppp)
 GO(SDL_LowerBlitScaled, iFpppp)
-GO(SDL_ltoa, pFlp)
-GO(SDL_malloc, pFu)
+GO(SDL_ltoa, pFlpi)
+GO(SDL_malloc, pFL)
 GO(SDL_MapRGB, uFpCCC)
 GO(SDL_MapRGBA, uFpCCCC)
 GO(SDL_MasksToPixelFormatEnum, uFiuuuu)
 GO(SDL_MaximizeWindow, vFp)
-GO(SDL_memcmp, iFppu)
+GO(SDL_memcmp, iFppL)
 GO(SDL_memcpy, pFppu)
 GO(SDL_memmove, pFppu)
 GO(SDL_MemoryBarrierAcquireFunction, vFv)
 GO(SDL_MemoryBarrierReleaseFunction, vFv)
-GO(SDL_memset, pFpiu)
+GO(SDL_memset, pFpiL)
 GO(SDL_MinimizeWindow, vFp)
 GO(SDL_MixAudio, vFppui)
 GO(SDL_MixAudioFormat, vFppWui)
 GO(SDL_MouseIsHaptic, iFv)
-GO(SDL_NewAudioStream, pFiCiiCi)
+GO(SDL_NewAudioStream, pFWCiWCi)
 GO(SDL_NumHaptics, iFv)
 GO(SDL_NumJoysticks, iFv)
 GO(SDL_NumSensors, iFv)
 GOM(SDL_OpenAudio, iFEpp)
 GOM(SDL_OpenAudioDevice, iFEpippi)
 GO(SDL_PauseAudio, vFi)
-GO(SDL_PauseAudioDevice, vFii)
-GO(SDL_PeepEvents, iFpiiuu)
-GO(SDL_PixelFormatEnumToMasks, iFuppppp)
+GO(SDL_PauseAudioDevice, vFui)
+GO(SDL_PeepEvents, iFpiuuu)
+GO(SDL_PixelFormatEnumToMasks, uFuppppp)
 GO(SDL_PollEvent, iFp)
 GO(SDL_pow, dFdd)
 GO(SDL_powf, fFff)
@@ -469,7 +469,7 @@ GO(SDL_PumpEvents, vFv)
 GO(SDL_PushEvent, iFp)
 GO2(SDL_qsort, vFEpuup, my_qsort)
 GO(SDL_QueryTexture, iFppppp)
-GO(SDL_QueueAudio, iFipu)
+GO(SDL_QueueAudio, iFupu)
 GO(SDL_Quit, vFv)
 GO(SDL_QuitSubSystem, vFu)
 GO(SDL_RaiseWindow, vFp)
@@ -480,13 +480,13 @@ GOM(SDL_ReadLE16, uFEp)
 GOM(SDL_ReadLE32, uFEp)
 GOM(SDL_ReadLE64, UFEp)
 GOM(SDL_ReadU8, uFEp)
-GO(SDL_realloc, pFpu)
-GO(SDL_RecordGesture, iFi)
+GO(SDL_realloc, pFpL)
+GO(SDL_RecordGesture, iFl)
 GO(SDL_RegisterEvents, uFi)
 GOM(SDL_RemoveTimer, iFEp)
 GO(SDL_RenderClear, iFp)
 GO(SDL_RenderCopy, iFpppp)
-GO(SDL_RenderCopyEx, iFppppdpi)
+GO(SDL_RenderCopyEx, iFppppdpu)
 // SDL_RenderCopyExF
 // SDL_RenderCopyF
 GO(SDL_RenderDrawLine, iFpiiii)
@@ -513,15 +513,16 @@ GO(SDL_RenderGetLogicalSize, vFppp)
 // SDL_RenderGetMetalLayer
 GO(SDL_RenderGetScale, vFppp)
 GO(SDL_RenderGetViewport, vFpp)
-GO(SDL_RenderIsClipEnabled, iFp)
+GO(SDL_RenderIsClipEnabled, uFp)
 GO(SDL_RenderPresent, vFp)
 GO(SDL_RenderReadPixels, iFppupi)
 GO(SDL_RenderSetClipRect, iFpp)
-GO(SDL_RenderSetIntegerScale, iFpi)
+GO(SDL_RenderSetIntegerScale, iFpu)
 GO(SDL_RenderSetLogicalSize, iFpii)
 GO(SDL_RenderSetScale, iFpff)
 GO(SDL_RenderSetViewport, iFpp)
 GO(SDL_RenderTargetSupported, iFp)
+GO(SDL_RenderWindowToLogical, vFpiipp)
 GO(SDL_ReportAssertion, uFpppi)
 GO(SDL_ResetAssertionReport, vFv)
 GO(SDL_ResetKeyboard, vFv)
@@ -533,8 +534,8 @@ GOM(SDL_RWFromMem, pFEpi)
 GOM(SDL_SaveAllDollarTemplates, iFEp)
 GOM(SDL_SaveBMP_RW, iFEppi)
 GOM(SDL_SaveDollarTemplate, iFEip)
-GO(SDL_scalbn, dFdl)
-GO(SDL_scalbnf, fFfl)
+GO(SDL_scalbn, dFdi)
+GO(SDL_scalbnf, fFfi)
 GO(SDL_SemPost, iFp)
 GO(SDL_SemTryWait, iFp)
 GO(SDL_SemValue, uFp)
@@ -555,17 +556,17 @@ GO(SDL_SemWaitTimeout, iFpu)
 // SDL_SensorUpdate
 // SDL_SetAssertionHandler
 GO(SDL_SetClipboardText, iFp)
-GO(SDL_SetClipRect, iFpp)
+GO(SDL_SetClipRect, uFpp)
 GO(SDL_SetColorKey, iFpiu)
 GO(SDL_SetCursor, vFp)
 GO(SDL_setenv, iFppi)
 GO(SDL_SetError, iFppppp)    // it use ..., so putarbitrary  4 args
 GOM(SDL_SetEventFilter, vFEpp)
-GO(SDL_SetHint, iFpp)
-GO(SDL_SetHintWithPriority, iFppi)
+GO(SDL_SetHint, uFpp)
+GO(SDL_SetHintWithPriority, uFppu)
 GO(SDL_SetMainReady, vFv)
 // SDL_SetMemoryFunctions
-GO(SDL_SetModState, vFi)
+GO(SDL_SetModState, vFu)
 GO(SDL_SetPaletteColors, iFppii)
 // SDL_SetPixelFormatPalette
 GO(SDL_SetRelativeMouseMode, iFi)
@@ -574,6 +575,7 @@ GO(SDL_SetRenderDrawColor, iFpCCCC)
 GO(SDL_SetRenderTarget, iFpp)
 GO(SDL_SetSurfaceAlphaMod, iFpC)
 GO(SDL_SetSurfaceBlendMode, iFpu)
+GO(SDL_SetSurfaceColorMod, iFpCCC)
 // SDL_SetSurfaceColorMod
 GO(SDL_SetSurfacePalette, iFpp)
 GO(SDL_SetSurfaceRLE, iFpi)
@@ -623,24 +625,24 @@ GO(SDL_strcasecmp, iFpp)
 GO(SDL_strchr, pFpi)
 GO(SDL_strcmp, iFpp)
 GO(SDL_strdup, pFp)
-GO(SDL_strlcat, uFppu)
+GO(SDL_strlcat, LFppL)
 GO(SDL_strlcpy, uFppu)
 GO(SDL_strlen, uFp)
 GO(SDL_strlwr, pFp)
-GO(SDL_strncasecmp, iFppu)
+GO(SDL_strncasecmp, iFppL)
 GO(SDL_strncmp, iFppu)
 GO(SDL_strrchr, pFpi)
 GO(SDL_strrev, pFp)
 GO(SDL_strstr, pFpp)
 GO(SDL_strtod, dFpp)
-GO(SDL_strtol, iFppi)
-GO(SDL_strtoll, IFppi)
-GO(SDL_strtoul, uFppi)
-GO(SDL_strtoull, UFppi)
+GO(SDL_strtol, lFppi)
+GO(SDL_strtoll, lFppi)
+GO(SDL_strtoul, LFppi)
+GO(SDL_strtoull, LFppi)
 GO(SDL_strupr, pFp)
 GO(SDL_tan, dFd)
 GO(SDL_tanf, fFf)
-GO(SDL_ThreadID, uFv)
+GO(SDL_ThreadID, LFv)
 GO(SDL_TLSCreate, uFv)
 GO(SDL_TLSGet, pFu)
 GOM(SDL_TLSSet, iFEupp)
@@ -648,8 +650,8 @@ GO(SDL_tolower, iFi)
 GO(SDL_toupper, iFi)
 GO(SDL_TryLockMutex, iFp)
 GO(SDL_uitoa, pFupi)
-GO(SDL_ulltoa, pFUpi)
-GO(SDL_ultoa, pFupi)
+GO(SDL_ulltoa, pFLpi)
+GO(SDL_ultoa, pFLpi)
 GO(SDL_UnionRect, vFppp)
 GOM(SDL_UnloadObject, vFEp)
 GO(SDL_UnlockAudio, vFv)
@@ -686,7 +688,7 @@ GO(SDL_WasInit, uFu)
 // SDL_wcsdup
 // SDL_wcslcat
 // SDL_wcslcpy
-GO(SDL_wcslen, uFp)
+GO(SDL_wcslen, LFp)
 GOM(SDL_WriteBE16, uFEpW)
 GOM(SDL_WriteBE32, uFEpu)
 GOM(SDL_WriteBE64, uFEpU)
